"""
Defines the unit tests for the
:mod:`colour.models.rgb.transfer_functions.log` module.
"""

import numpy as np
import unittest

from colour.models.rgb.transfer_functions import (
    logarithmic_function_basic,
    logarithmic_function_quasilog,
    logarithmic_function_camera,
    log_encoding_Log2,
    log_decoding_Log2,
)
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestLogarithmFunction_Basic",
    "TestLogarithmFunction_Quasilog",
    "TestLogarithmFunction_Camera",
    "TestLogEncoding_Log2",
    "TestLogDecoding_Log2",
]


class TestLogarithmFunction_Basic(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_basic` definition unit tests methods.
    """

    def test_logarithmic_function_basic(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_basic` definition.
        """

        self.assertAlmostEqual(
            logarithmic_function_basic(0.18), -2.473931188332412, places=7
        )

        self.assertAlmostEqual(
            logarithmic_function_basic(-2.473931188332412, "antiLog2"),
            0.180000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_basic(0.18, "log10"),
            -0.744727494896694,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_basic(-0.744727494896694, "antiLog10"),
            0.179999999999999,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_basic(0.18, "logB", 3),
            -1.560876795007312,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_basic(-1.560876795007312, "antiLogB", 3),
            0.180000000000000,
            places=7,
        )

    def test_n_dimensional_logarithmic_function_basic(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_basic` definition n-dimensional arrays support.
        """

        styles = ["log10", "antiLog10", "log2", "antiLog2", "logB", "antiLogB"]

        for style in styles:
            a = 0.18
            a_p = logarithmic_function_basic(a, style)

            a = np.tile(a, 6)
            a_p = np.tile(a_p, 6)
            np.testing.assert_almost_equal(
                logarithmic_function_basic(a, style), a_p, decimal=7
            )

            a = np.reshape(a, (2, 3))
            a_p = np.reshape(a_p, (2, 3))
            np.testing.assert_almost_equal(
                logarithmic_function_basic(a, style), a_p, decimal=7
            )

            a = np.reshape(a, (2, 3, 1))
            a_p = np.reshape(a_p, (2, 3, 1))
            np.testing.assert_almost_equal(
                logarithmic_function_basic(a, style), a_p, decimal=7
            )

    @ignore_numpy_errors
    def test_nan_logarithmic_function_basic(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_basic` definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        styles = ["log10", "antiLog10", "log2", "antiLog2", "logB", "antiLogB"]

        for case in cases:
            for style in styles:
                logarithmic_function_basic(case, style)


class TestLogarithmFunction_Quasilog(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_quasilog` definition unit tests methods.
    """

    def test_logarithmic_function_quasilog(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_quasilog` definition.
        """

        self.assertAlmostEqual(
            logarithmic_function_quasilog(0.18), -2.473931188332412, places=7
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(-2.473931188332412, "logToLin"),
            0.18,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(0.18, "linToLog", 10),
            -0.744727494896694,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(-0.744727494896694, "logToLin", 10),
            0.18,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(0.18, "linToLog", 10, 0.75),
            -0.558545621172520,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(
                -0.558545621172520, "logToLin", 10, 0.75
            ),
            0.18,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(0.18, "linToLog", 10, 0.75, 0.75),
            -0.652249673628745,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(
                -0.652249673628745, "logToLin", 10, 0.75, 0.75
            ),
            0.18,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(
                0.18, "linToLog", 10, 0.75, 0.75, 0.001
            ),
            -0.651249673628745,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(
                -0.651249673628745, "logToLin", 10, 0.75, 0.75, 0.001
            ),
            0.18,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(
                0.18, "linToLog", 10, 0.75, 0.75, 0.001, 0.01
            ),
            -0.627973998323769,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_quasilog(
                -0.627973998323769, "logToLin", 10, 0.75, 0.75, 0.001, 0.01
            ),
            0.18,
            places=7,
        )

    def test_n_dimensional_logarithmic_function_quasilog(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_quasilog` definition n-dimensional arrays support.
        """

        styles = ["lintolog", "logtolin"]

        for style in styles:
            a = 0.18
            a_p = logarithmic_function_quasilog(a, style)

            a = np.tile(a, 6)
            a_p = np.tile(a_p, 6)
            np.testing.assert_almost_equal(
                logarithmic_function_quasilog(a, style), a_p, decimal=7
            )

            a = np.reshape(a, (2, 3))
            a_p = np.reshape(a_p, (2, 3))
            np.testing.assert_almost_equal(
                logarithmic_function_quasilog(a, style), a_p, decimal=7
            )

            a = np.reshape(a, (2, 3, 1))
            a_p = np.reshape(a_p, (2, 3, 1))
            np.testing.assert_almost_equal(
                logarithmic_function_quasilog(a, style), a_p, decimal=7
            )

    @ignore_numpy_errors
    def test_nan_logarithmic_function_quasilog(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_quasilog` definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        styles = ["lintolog", "logtolin"]

        for case in cases:
            for style in styles:
                logarithmic_function_quasilog(case, style)


class TestLogarithmFunction_Camera(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_camera` definition unit tests methods.
    """

    def test_logarithmic_function_camera(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_camera` definition.
        """

        self.assertAlmostEqual(
            logarithmic_function_camera(0, "cameraLinToLog"),
            -9.08655123066369,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(-9.08655123066369, "cameraLogToLin"),
            0.000000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(0.18, "cameraLinToLog"),
            -2.473931188332412,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(-2.473931188332412, "cameraLogToLin"),
            0.180000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(1, "cameraLinToLog"),
            0.000000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(0, "cameraLogToLin"),
            1.000000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(0.18, "cameraLinToLog", 10),
            -0.744727494896693,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                -0.744727494896693, "cameraLogToLin", 10
            ),
            0.180000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(0.18, "cameraLinToLog", 10, 0.25),
            -0.186181873724173,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                -0.186181873724173, "cameraLogToLin", 10, 0.25
            ),
            0.180000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.18, "cameraLinToLog", 10, 0.25, 0.95
            ),
            -0.191750972401961,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                -0.191750972401961, "cameraLogToLin", 10, 0.25, 0.95
            ),
            0.180000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.18, "cameraLinToLog", 10, 0.25, 0.95, 0.6
            ),
            0.408249027598038,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.408249027598038, "cameraLogToLin", 10, 0.25, 0.95, 0.6
            ),
            0.179999999999999,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.18, "cameraLinToLog", 10, 0.25, 0.95, 0.6, 0.01
            ),
            0.414419643717296,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.414419643717296, "cameraLogToLin", 10, 0.25, 0.95, 0.6, 0.01
            ),
            0.180000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.005, "cameraLinToLog", 10, 0.25, 0.95, 0.6, 0.01, 0.01
            ),
            0.146061232468316,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.146061232468316,
                "cameraLogToLin",
                10,
                0.25,
                0.95,
                0.6,
                0.01,
                0.01,
            ),
            0.005000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.005, "cameraLinToLog", 10, 0.25, 0.95, 0.6, 0.01, 0.01, 6
            ),
            0.142508652840630,
            places=7,
        )

        self.assertAlmostEqual(
            logarithmic_function_camera(
                0.142508652840630,
                "cameraLogToLin",
                10,
                0.25,
                0.95,
                0.6,
                0.01,
                0.01,
                6,
            ),
            0.005000000000000,
            places=7,
        )

    def test_n_dimensional_logarithmic_function_camera(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_camera` definition n-dimensional arrays support.
        """

        styles = ["cameraLinToLog", "cameraLogToLin"]

        for style in styles:
            a = 0.18
            a_p = logarithmic_function_camera(a, style)

            a = np.tile(a, 6)
            a_p = np.tile(a_p, 6)
            np.testing.assert_almost_equal(
                logarithmic_function_camera(a, style), a_p, decimal=7
            )

            a = np.reshape(a, (2, 3))
            a_p = np.reshape(a_p, (2, 3))
            np.testing.assert_almost_equal(
                logarithmic_function_camera(a, style), a_p, decimal=7
            )

            a = np.reshape(a, (2, 3, 1))
            a_p = np.reshape(a_p, (2, 3, 1))
            np.testing.assert_almost_equal(
                logarithmic_function_camera(a, style), a_p, decimal=7
            )

    @ignore_numpy_errors
    def test_nan_logarithmic_function_camera(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
logarithmic_function_camera` definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        styles = ["cameraLinToLog", "cameraLogToLin"]

        for case in cases:
            for style in styles:
                logarithmic_function_camera(case, style)


class TestLogEncoding_Log2(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.log.\
log_encoding_Log2` definition unit tests methods.
    """

    def test_log_encoding_Log2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
log_encoding_Log2` definition.
        """

        self.assertAlmostEqual(log_encoding_Log2(0.0), -np.inf, places=7)

        self.assertAlmostEqual(log_encoding_Log2(0.18), 0.5, places=7)

        self.assertAlmostEqual(
            log_encoding_Log2(1.0), 0.690302399102493, places=7
        )

        self.assertAlmostEqual(
            log_encoding_Log2(0.18, 0.12), 0.544997115440089, places=7
        )

        self.assertAlmostEqual(
            log_encoding_Log2(0.18, 0.12, 2**-10),
            0.089857490719529,
            places=7,
        )

        self.assertAlmostEqual(
            log_encoding_Log2(0.18, 0.12, 2**-10, 2**10),
            0.000570299311674,
            places=7,
        )

    def test_n_dimensional_log_encoding_Log2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
log_encoding_Log2` definition n-dimensional arrays support.
        """

        x = 0.18
        y = log_encoding_Log2(x)

        x = np.tile(x, 6)
        y = np.tile(y, 6)
        np.testing.assert_almost_equal(log_encoding_Log2(x), y, decimal=7)

        x = np.reshape(x, (2, 3))
        y = np.reshape(y, (2, 3))
        np.testing.assert_almost_equal(log_encoding_Log2(x), y, decimal=7)

        x = np.reshape(x, (2, 3, 1))
        y = np.reshape(y, (2, 3, 1))
        np.testing.assert_almost_equal(log_encoding_Log2(x), y, decimal=7)

    def test_domain_range_scale_log_encoding_Log2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
log_encoding_Log2` definition domain and range scale support.
        """

        x = 0.18
        y = log_encoding_Log2(x)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    log_encoding_Log2(x * factor), y * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_log_encoding_Log2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
log_encoding_Log2` definition nan support.
        """

        log_encoding_Log2(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestLogDecoding_Log2(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.log.\
log_decoding_Log2` definition unit tests methods.
    """

    def test_log_decoding_Log2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
log_decoding_Log2` definition.
        """

        self.assertAlmostEqual(
            log_decoding_Log2(0.0), 0.001988737822087, places=7
        )

        self.assertAlmostEqual(log_decoding_Log2(0.5), 0.18, places=7)

        self.assertAlmostEqual(
            log_decoding_Log2(0.690302399102493), 1.0, places=7
        )

        self.assertAlmostEqual(
            log_decoding_Log2(0.544997115440089, 0.12), 0.18, places=7
        )

        self.assertAlmostEqual(
            log_decoding_Log2(0.089857490719529, 0.12, 2**-10),
            0.180000000000000,
            places=7,
        )

        self.assertAlmostEqual(
            log_decoding_Log2(0.000570299311674, 0.12, 2**-10, 2**10),
            0.180000000000000,
            places=7,
        )

    def test_n_dimensional_log_decoding_Log2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
log_decoding_Log2` definition n-dimensional arrays support.
        """

        y = 0.5
        x = log_decoding_Log2(y)

        y = np.tile(y, 6)
        x = np.tile(x, 6)
        np.testing.assert_almost_equal(log_decoding_Log2(y), x, decimal=7)

        y = np.reshape(y, (2, 3))
        x = np.reshape(x, (2, 3))
        np.testing.assert_almost_equal(log_decoding_Log2(y), x, decimal=7)

        y = np.reshape(y, (2, 3, 1))
        x = np.reshape(x, (2, 3, 1))
        np.testing.assert_almost_equal(log_decoding_Log2(y), x, decimal=7)

    def test_domain_range_scale_log_decoding_Log2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
log_decoding_Log2` definition domain and range scale support.
        """

        y = 0.5
        x = log_decoding_Log2(y)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    log_decoding_Log2(y * factor), x * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_log_decoding_Log2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.log.\
log_decoding_Log2` definition nan support.
        """

        log_decoding_Log2(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


if __name__ == "__main__":
    unittest.main()



"""Defines the unit tests for the :mod:`colour.models.ipt` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.models import XYZ_to_IPT, IPT_to_XYZ, IPT_hue_angle
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestXYZ_to_IPT",
    "TestIPT_to_XYZ",
    "TestIPTHueAngle",
]


class TestXYZ_to_IPT(unittest.TestCase):
    """Define :func:`colour.models.ipt.XYZ_to_IPT` definition unit tests methods."""

    def test_XYZ_to_IPT(self):
        """Test :func:`colour.models.ipt.XYZ_to_IPT` definition."""

        np.testing.assert_almost_equal(
            XYZ_to_IPT(np.array([0.20654008, 0.12197225, 0.05136952])),
            np.array([0.38426191, 0.38487306, 0.18886838]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_IPT(np.array([0.14222010, 0.23042768, 0.10495772])),
            np.array([0.49437481, -0.19251742, 0.18080304]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_IPT(np.array([0.07818780, 0.06157201, 0.28099326])),
            np.array([0.35167774, -0.07525627, -0.30921279]),
            decimal=7,
        )

    def test_n_dimensional_XYZ_to_IPT(self):
        """
        Test :func:`colour.models.ipt.XYZ_to_IPT` definition n-dimensional
        support.
        """

        XYZ = np.array([0.20654008, 0.12197225, 0.05136952])
        IPT = XYZ_to_IPT(XYZ)

        XYZ = np.tile(XYZ, (6, 1))
        IPT = np.tile(IPT, (6, 1))
        np.testing.assert_almost_equal(XYZ_to_IPT(XYZ), IPT, decimal=7)

        XYZ = np.reshape(XYZ, (2, 3, 3))
        IPT = np.reshape(IPT, (2, 3, 3))
        np.testing.assert_almost_equal(XYZ_to_IPT(XYZ), IPT, decimal=7)

    def test_domain_range_scale_XYZ_to_IPT(self):
        """
        Test :func:`colour.models.ipt.XYZ_to_IPT` definition domain and
        range scale support.
        """

        XYZ = np.array([0.20654008, 0.12197225, 0.05136952])
        IPT = XYZ_to_IPT(XYZ)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    XYZ_to_IPT(XYZ * factor), IPT * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_XYZ_to_IPT(self):
        """Test :func:`colour.models.ipt.XYZ_to_IPT` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            XYZ = np.array(case)
            XYZ_to_IPT(XYZ)


class TestIPT_to_XYZ(unittest.TestCase):
    """
    Define :func:`colour.models.ipt.IPT_to_XYZ` definition unit tests
    methods.
    """

    def test_IPT_to_XYZ(self):
        """Test :func:`colour.models.ipt.IPT_to_XYZ` definition."""

        np.testing.assert_almost_equal(
            IPT_to_XYZ(np.array([0.38426191, 0.38487306, 0.18886838])),
            np.array([0.20654008, 0.12197225, 0.05136952]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            IPT_to_XYZ(np.array([0.49437481, -0.19251742, 0.18080304])),
            np.array([0.14222010, 0.23042768, 0.10495772]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            IPT_to_XYZ(np.array([0.35167774, -0.07525627, -0.30921279])),
            np.array([0.07818780, 0.06157201, 0.28099326]),
            decimal=7,
        )

    def test_n_dimensional_IPT_to_XYZ(self):
        """
        Test :func:`colour.models.ipt.IPT_to_XYZ` definition n-dimensional
        support.
        """

        IPT = np.array([0.38426191, 0.38487306, 0.18886838])
        XYZ = IPT_to_XYZ(IPT)

        IPT = np.tile(IPT, (6, 1))
        XYZ = np.tile(XYZ, (6, 1))
        np.testing.assert_almost_equal(IPT_to_XYZ(IPT), XYZ, decimal=7)

        IPT = np.reshape(IPT, (2, 3, 3))
        XYZ = np.reshape(XYZ, (2, 3, 3))
        np.testing.assert_almost_equal(IPT_to_XYZ(IPT), XYZ, decimal=7)

    def test_domain_range_scale_IPT_to_XYZ(self):
        """
        Test :func:`colour.models.ipt.IPT_to_XYZ` definition domain and
        range scale support.
        """

        IPT = np.array([0.38426191, 0.38487306, 0.18886838])
        XYZ = IPT_to_XYZ(IPT)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    IPT_to_XYZ(IPT * factor), XYZ * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_IPT_to_XYZ(self):
        """Test :func:`colour.models.ipt.IPT_to_XYZ` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            IPT = np.array(case)
            IPT_to_XYZ(IPT)


class TestIPTHueAngle(unittest.TestCase):
    """
    Define :func:`colour.models.ipt.IPT_hue_angle` definition unit tests
    methods.
    """

    def test_IPT_hue_angle(self):
        """Test :func:`colour.models.ipt.IPT_hue_angle` definition."""

        np.testing.assert_almost_equal(
            IPT_hue_angle(np.array([0.20654008, 0.12197225, 0.05136952])),
            22.838754548625527,
            decimal=7,
        )

        np.testing.assert_almost_equal(
            IPT_hue_angle(np.array([0.14222010, 0.23042768, 0.10495772])),
            24.488834912466245,
            decimal=7,
        )

        np.testing.assert_almost_equal(
            IPT_hue_angle(np.array([0.07818780, 0.06157201, 0.28099326])),
            77.640533743711813,
            decimal=7,
        )

    def test_n_dimensional_IPT_hue_angle(self):
        """
        Test :func:`colour.models.ipt.IPT_hue_angle` definition n-dimensional
        support.
        """

        IPT = np.array([0.20654008, 0.12197225, 0.05136952])
        hue = IPT_hue_angle(IPT)

        IPT = np.tile(IPT, (6, 1))
        hue = np.tile(hue, 6)
        np.testing.assert_almost_equal(IPT_hue_angle(IPT), hue, decimal=7)

        IPT = np.reshape(IPT, (2, 3, 3))
        hue = np.reshape(hue, (2, 3))
        np.testing.assert_almost_equal(IPT_hue_angle(IPT), hue, decimal=7)

    def test_domain_range_scale_IPT_hue_angle(self):
        """
        Test :func:`colour.models.ipt.IPT_hue_angle` definition domain and
        range scale support.
        """

        IPT = np.array([0.20654008, 0.12197225, 0.05136952])
        hue = IPT_hue_angle(IPT)

        d_r = (("reference", 1, 1), ("1", 1, 1 / 360), ("100", 100, 1 / 3.6))
        for scale, factor_a, factor_b in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    IPT_hue_angle(IPT * factor_a), hue * factor_b, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_IPT_hue_angle(self):
        """Test :func:`colour.models.ipt.IPT_hue_angle` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            IPT = np.array(case)
            IPT_hue_angle(IPT)


if __name__ == "__main__":
    unittest.main()



"""Defines unit tests for :mod:`colour.models.rgb.hanbury2003` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.models.rgb import RGB_to_IHLS, IHLS_to_RGB
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestRGB_to_IHLS",
    "TestIHLS_to_RGB",
]


class TestRGB_to_IHLS(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.hanbury2003.RGB_to_IHLS` definition unit
    tests methods.
    """

    def test_RGB_to_IHLS(self):
        """Test :func:`colour.models.rgb.hanbury2003.RGB_to_IHLS` definition."""

        np.testing.assert_almost_equal(
            RGB_to_IHLS(np.array([0.45620519, 0.03081071, 0.04091952])),
            np.array([6.26236117, 0.12197943, 0.42539448]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            RGB_to_IHLS(np.array([0.00000000, 0.00000000, 0.00000000])),
            np.array([0.00000000, 0.00000000, 0.00000000]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            RGB_to_IHLS(np.array([1.00000000, 1.00000000, 1.00000000])),
            np.array([0.00000000, 1.00000000, 0.00000000]),
            decimal=7,
        )

    def test_n_dimensional_RGB_to_IHLS(self):
        """
        Test :func:`colour.models.rgb.hanbury2003.RGB_to_IHLS` definition
        n-dimensional arrays support.
        """

        RGB = np.array([0.45620519, 0.03081071, 0.04091952])
        HYS = RGB_to_IHLS(RGB)

        RGB = np.tile(RGB, (6, 1))
        HYS = np.tile(HYS, (6, 1))
        np.testing.assert_almost_equal(RGB_to_IHLS(RGB), HYS, decimal=7)

        RGB = np.reshape(RGB, (2, 3, 3))
        HYS = np.reshape(HYS, (2, 3, 3))
        np.testing.assert_almost_equal(RGB_to_IHLS(RGB), HYS, decimal=7)

    def test_domain_range_scale_RGB_to_IHLS(self):
        """
        Test :func:`colour.models.rgb.hanbury2003.RGB_to_IHLS` definition
        domain and range scale support.
        """

        RGB = np.array([0.45620519, 0.03081071, 0.04091952])
        HYS = RGB_to_IHLS(RGB)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    RGB_to_IHLS(RGB * factor), HYS * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_RGB_to_IHLS(self):
        """
        Test :func:`colour.models.rgb.hanbury2003.RGB_to_IHLS` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            RGB = np.array(case)
            RGB_to_IHLS(RGB)


class TestIHLS_to_RGB(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.hanbury2003.RGB_to_IHLS` definition unit
    tests methods.
    """

    def test_IHLS_to_RGB(self):
        """Test :func:`colour.models.rgb.hanbury2003.IHLS_to_RGB` definition."""

        np.testing.assert_almost_equal(
            IHLS_to_RGB(np.array([6.26236117, 0.12197943, 0.42539448])),
            np.array([0.45620519, 0.03081071, 0.04091952]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            IHLS_to_RGB(np.array([0.00000000, 0.00000000, 0.00000000])),
            np.array([0.00000000, 0.00000000, 0.00000000]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            IHLS_to_RGB(np.array([0.00000000, 1.00000000, 0.00000000])),
            np.array([1.00000000, 1.00000000, 1.00000000]),
            decimal=7,
        )

    def test_n_dimensional_IHLS_to_RGB(self):
        """
        Test :func:`colour.models.rgb.hanbury2003.IHLS_to_RGB` definition
        n-dimensional arrays support.
        """

        HYS = np.array([6.26236117, 0.12197943, 0.42539448])
        RGB = IHLS_to_RGB(HYS)

        HYS = np.tile(HYS, (6, 1))
        RGB = np.tile(RGB, (6, 1))
        np.testing.assert_almost_equal(IHLS_to_RGB(HYS), RGB, decimal=7)

        HYS = np.reshape(HYS, (2, 3, 3))
        RGB = np.reshape(RGB, (2, 3, 3))
        np.testing.assert_almost_equal(IHLS_to_RGB(HYS), RGB, decimal=7)

    def test_domain_range_scale_IHLS_to_RGB(self):
        """
        Test :func:`colour.models.rgb.hanbury2003.IHLS_to_RGB` definition
        domain and range scale support.
        """

        HYS = np.array([6.26236117, 0.12197943, 0.42539448])
        RGB = IHLS_to_RGB(HYS)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    IHLS_to_RGB(HYS * factor), RGB * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_IHLS_to_RGB(self):
        """
        Test :func:`colour.models.rgb.hanbury2003.IHLS_to_RGB` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            HYS = np.array(case)
            IHLS_to_RGB(HYS)


if __name__ == "__main__":
    unittest.main()



"""Showcases *Fairchild (1990)* chromatic adaptation model computations."""

import numpy as np

import colour
from colour.utilities import message_box

message_box('"Fairchild (1990)" Chromatic Adaptation Model Computations')

XYZ_1 = np.array([0.1953, 0.2307, 0.2497])
XYZ_n = np.array([1.1115, 1.0000, 0.3520])
XYZ_r = np.array([0.9481, 1.0000, 1.0730])
Y_n = 200
message_box(
    f'Computing chromatic adaptation using "Fairchild (1990)" chromatic '
    f"adaptation model.\n\n"
    f'\t"XYZ_1": {XYZ_1}\n'
    f'\t"XYZ_n": {XYZ_n}\n'
    f'\t"XYZ_r": {XYZ_r}\n'
    f'\t"Y_n": {Y_n}'
)
print(
    colour.chromatic_adaptation(
        XYZ_1, XYZ_n, XYZ_r, method="Fairchild 1990", Y_n=Y_n
    )
)
print(
    colour.adaptation.chromatic_adaptation_Fairchild1990(
        XYZ_1 * 100, XYZ_n, XYZ_r, Y_n
    )
    / 100
)



"""Showcases *RGB* colourspace derivation."""

import numpy as np

import colour
from colour.utilities import message_box

message_box('"RGB" Colourspace Derivation')

primaries = np.array(
    [
        [0.73470, 0.26530],
        [0.00000, 1.00000],
        [0.00010, -0.07700],
    ]
)
whitepoint = np.array([0.32168, 0.33767])
message_box(
    f'Computing the normalised primary matrix for the "ACES2065-1" colourspace '
    f'transforming from the "ACES2065-1" colourspace to "CIE XYZ" tristimulus '
    f"values using user defined primaries matrix and whitepoint:\n\n"
    f"\t{primaries[0]}\n"
    f"\t{primaries[1]}\n"
    f"\t{primaries[2]}\n"
    f"\t{whitepoint}"
)
print(colour.normalised_primary_matrix(primaries, whitepoint))

print("\n")

message_box(
    'Computing the normalised primary matrix for the "ACES2065-1" colourspace '
    'transforming from the "ACES2065-1" colourspace to "CIE XYZ" tristimulus '
    "values using colour models dataset."
)
print(
    colour.normalised_primary_matrix(
        colour.RGB_COLOURSPACES["ACES2065-1"].primaries,
        colour.RGB_COLOURSPACES["ACES2065-1"].whitepoint,
    )
)

print("\n")

message_box(
    'Computing the normalised primary matrix for the "ACES2065-1" colourspace '
    'transforming from "CIE XYZ" tristimulus values to the "ACES2065-1" '
    "colourspace using colour models dataset."
)
print(
    np.linalg.inv(
        colour.normalised_primary_matrix(
            colour.RGB_COLOURSPACES["ACES2065-1"].primaries,
            colour.RGB_COLOURSPACES["ACES2065-1"].whitepoint,
        )
    )
)

print("\n")

message_box(
    'Computing the "sRGB" colourspace primaries chromatically adapted to the '
    '"CIE Standard Illuminant D50":\n'
)
print(
    colour.chromatically_adapted_primaries(
        colour.RGB_COLOURSPACES["sRGB"].primaries,
        colour.RGB_COLOURSPACES["sRGB"].whitepoint,
        colour.CCS_ILLUMINANTS["CIE 1931 2 Degree Standard Observer"]["D50"],
    )
)

print("\n")

npm = np.array(
    [
        [0.41240000, 0.35760000, 0.18050000],
        [0.21260000, 0.71520000, 0.07220000],
        [0.01930000, 0.11920000, 0.95050000],
    ]
)
message_box(
    f"Computing the primaries and whitepoint from given normalised primary "
    f"matrix:\n\n{npm}"
)
print(colour.primaries_whitepoint(npm))

print("\n")

RGB = np.array([0.45620519, 0.03081071, 0.04091952])
message_box(f'Computing the "RGB" luminance of given "RGB" values:\n\n\t{RGB}')
print(
    colour.RGB_luminance(
        RGB,
        colour.RGB_COLOURSPACES["sRGB"].primaries,
        colour.RGB_COLOURSPACES["sRGB"].whitepoint,
    )
)

print("\n")

message_box(
    f'Computing the "RGB" luminance equation of given "RGB" values:\n\n\t{RGB}'
)
print(
    colour.RGB_luminance(
        RGB,
        colour.RGB_COLOURSPACES["sRGB"].primaries,
        colour.RGB_COLOURSPACES["sRGB"].whitepoint,
    )
)



"""
Sony .spimtx LUT Format Input / Output Utilities
================================================

Defines *Sony* *.spimtx* *LUT* format related input / output utilities objects.

-   :func:`colour.io.read_LUT_SonySPImtx`
-   :func:`colour.io.write_LUT_SonySPImtx`
"""

from __future__ import annotations

import numpy as np

from colour.constants import DEFAULT_FLOAT_DTYPE
from colour.hints import Integer
from colour.io.luts.common import path_to_title
from colour.io.luts import LUTOperatorMatrix
from colour.hints import Boolean

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "read_LUT_SonySPImtx",
    "write_LUT_SonySPImtx",
]


def read_LUT_SonySPImtx(path: str) -> LUTOperatorMatrix:
    """
    Read given *Sony* *.spimtx* *LUT* file.

    Parameters
    ----------
    path
        *LUT* path.

    Returns
    -------
    :class:`colour.LUTOperatorMatrix`
        :class:`colour.io.Matrix` class instance.

    Examples
    --------
    >>> import os
    >>> path = os.path.join(
    ...     os.path.dirname(__file__), 'tests', 'resources', 'sony_spimtx',
    ...     'dt.spimtx')
    >>> print(read_LUT_SonySPImtx(path))
    LUTOperatorMatrix - dt
    ----------------------
    <BLANKLINE>
    Matrix     : [[ 0.864274  0.        0.        0.      ]
                  [ 0.        0.864274  0.        0.      ]
                  [ 0.        0.        0.864274  0.      ]
                  [ 0.        0.        0.        1.      ]]
    Offset     : [ 0.  0.  0.  0.]
    """

    matrix = np.loadtxt(path, dtype=DEFAULT_FLOAT_DTYPE)
    matrix = np.reshape(matrix, (3, 4))
    offset = matrix[:, 3] / 65535
    matrix = matrix[:3, :3]

    title = path_to_title(path)

    return LUTOperatorMatrix(matrix, offset, name=title)


def write_LUT_SonySPImtx(
    LUT: LUTOperatorMatrix, path: str, decimals: Integer = 7
) -> Boolean:
    """
    Write given *LUT* to given *Sony* *.spimtx* *LUT* file.

    Parameters
    ----------
    LUT
        :class:`colour.LUTOperatorMatrix` class instance to write at given
        path.
    path
        *LUT* path.
    decimals
        Formatting decimals.

    Returns
    -------
    :class:`bool`
        Definition success.

    Examples
    --------
    >>> matrix = np.array([[ 1.45143932, -0.23651075, -0.21492857],
    ...                    [-0.07655377,  1.1762297 , -0.09967593],
    ...                    [ 0.00831615, -0.00603245,  0.9977163 ]])
    >>> M = LUTOperatorMatrix(matrix)
    >>> write_LUT_SonySPI1D(M, 'My_LUT.spimtx')  # doctest: +SKIP
    """

    matrix, offset = LUT.matrix, LUT.offset
    offset *= 65535

    array = np.hstack(
        [
            np.reshape(matrix, (4, 4))[:3, :3],
            np.transpose(np.array([offset[:3]])),
        ]
    )

    np.savetxt(path, array, fmt=f"%.{decimals}f")

    return True



"""Defines the unit tests for the :mod:`colour.models.cie_ucs` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.models import (
    XYZ_to_UCS,
    UCS_to_XYZ,
    UCS_to_uv,
    uv_to_UCS,
    UCS_uv_to_xy,
    xy_to_UCS_uv,
)
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestXYZ_to_UCS",
    "TestUCS_to_XYZ",
    "TestUCS_to_uv",
    "Testuv_to_UCS",
    "TestUCS_uv_to_xy",
    "TestXy_to_UCS_uv",
]


class TestXYZ_to_UCS(unittest.TestCase):
    """
    Define :func:`colour.models.cie_ucs.XYZ_to_UCS` definition unit tests
    methods.
    """

    def test_XYZ_to_UCS(self):
        """Test :func:`colour.models.cie_ucs.XYZ_to_UCS` definition."""

        np.testing.assert_almost_equal(
            XYZ_to_UCS(np.array([0.20654008, 0.12197225, 0.05136952])),
            np.array([0.13769339, 0.12197225, 0.10537310]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_UCS(np.array([0.14222010, 0.23042768, 0.10495772])),
            np.array([0.09481340, 0.23042768, 0.32701033]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_UCS(np.array([0.07818780, 0.06157201, 0.28099326])),
            np.array([0.05212520, 0.06157201, 0.19376075]),
            decimal=7,
        )

    def test_n_dimensional_XYZ_to_UCS(self):
        """
        Test :func:`colour.models.cie_ucs.XYZ_to_UCS` definition n-dimensional
        support.
        """

        XYZ = np.array([0.20654008, 0.12197225, 0.05136952])
        UCS = XYZ_to_UCS(XYZ)

        UCS = np.tile(UCS, (6, 1))
        XYZ = np.tile(XYZ, (6, 1))
        np.testing.assert_almost_equal(XYZ_to_UCS(XYZ), UCS, decimal=7)

        UCS = np.reshape(UCS, (2, 3, 3))
        XYZ = np.reshape(XYZ, (2, 3, 3))
        np.testing.assert_almost_equal(XYZ_to_UCS(XYZ), UCS, decimal=7)

    def test_domain_range_scale_XYZ_to_UCS(self):
        """
        Test :func:`colour.models.cie_ucs.XYZ_to_UCS` definition domain and
        range scale support.
        """

        XYZ = np.array([0.0704953400, 0.1008000000, 0.0955831300])
        UCS = XYZ_to_UCS(XYZ)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    XYZ_to_UCS(XYZ * factor), UCS * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_XYZ_to_UCS(self):
        """Test :func:`colour.models.cie_ucs.XYZ_to_UCS` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            XYZ = np.array(case)
            XYZ_to_UCS(XYZ)


class TestUCS_to_XYZ(unittest.TestCase):
    """
    Define :func:`colour.models.cie_ucs.UCS_to_XYZ` definition unit tests
    methods.
    """

    def test_UCS_to_XYZ(self):
        """Test :func:`colour.models.cie_ucs.UCS_to_XYZ` definition."""

        np.testing.assert_almost_equal(
            UCS_to_XYZ(np.array([0.13769339, 0.12197225, 0.10537310])),
            np.array([0.20654008, 0.12197225, 0.05136952]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            UCS_to_XYZ(np.array([0.09481340, 0.23042768, 0.32701033])),
            np.array([0.14222010, 0.23042768, 0.10495772]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            UCS_to_XYZ(np.array([0.05212520, 0.06157201, 0.19376075])),
            np.array([0.07818780, 0.06157201, 0.28099326]),
            decimal=7,
        )

    def test_n_dimensional_UCS_to_XYZ(self):
        """
        Test :func:`colour.models.cie_ucs.UCS_to_XYZ` definition n-dimensional
        support.
        """

        UCS = np.array([0.13769339, 0.12197225, 0.10537310])
        XYZ = UCS_to_XYZ(UCS)

        UCS = np.tile(UCS, (6, 1))
        XYZ = np.tile(XYZ, (6, 1))
        np.testing.assert_almost_equal(UCS_to_XYZ(UCS), XYZ, decimal=7)

        UCS = np.reshape(UCS, (2, 3, 3))
        XYZ = np.reshape(XYZ, (2, 3, 3))
        np.testing.assert_almost_equal(UCS_to_XYZ(UCS), XYZ, decimal=7)

    def test_domain_range_scale_UCS_to_XYZ(self):
        """
        Test :func:`colour.models.cie_ucs.UCS_to_XYZ` definition domain and
        range scale support.
        """

        UCS = np.array([0.0469968933, 0.1008000000, 0.1637438950])
        XYZ = UCS_to_XYZ(UCS)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    UCS_to_XYZ(UCS * factor), XYZ * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_UCS_to_XYZ(self):
        """Test :func:`colour.models.cie_ucs.UCS_to_XYZ` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            UCS = np.array(case)
            UCS_to_XYZ(UCS)


class TestUCS_to_uv(unittest.TestCase):
    """
    Define :func:`colour.models.cie_ucs.UCS_to_uv` definition unit tests
    methods.
    """

    def test_UCS_to_uv(self):
        """Test :func:`colour.models.cie_ucs.UCS_to_uv` definition."""

        np.testing.assert_almost_equal(
            UCS_to_uv(np.array([0.13769339, 0.12197225, 0.10537310])),
            np.array([0.37720213, 0.33413508]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            UCS_to_uv(np.array([0.09481340, 0.23042768, 0.32701033])),
            np.array([0.14536327, 0.35328046]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            UCS_to_uv(np.array([0.05212520, 0.06157201, 0.19376075])),
            np.array([0.16953602, 0.20026156]),
            decimal=7,
        )

    def test_n_dimensional_UCS_to_uv(self):
        """
        Test :func:`colour.models.cie_ucs.UCS_to_uv` definition n-dimensional
        support.
        """

        UCS = np.array([0.13769339, 0.12197225, 0.10537310])
        uv = UCS_to_uv(UCS)

        UCS = np.tile(UCS, (6, 1))
        uv = np.tile(uv, (6, 1))
        np.testing.assert_almost_equal(UCS_to_uv(UCS), uv, decimal=7)

        UCS = np.reshape(UCS, (2, 3, 3))
        uv = np.reshape(uv, (2, 3, 2))
        np.testing.assert_almost_equal(UCS_to_uv(UCS), uv, decimal=7)

    def test_domain_range_scale_UCS_to_uv(self):
        """
        Test :func:`colour.models.cie_ucs.UCS_to_uv` definition domain and
        range scale support.
        """

        UCS = np.array([0.0469968933, 0.1008000000, 0.1637438950])
        uv = UCS_to_uv(UCS)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    UCS_to_uv(UCS * factor), uv, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_UCS_to_uv(self):
        """Test :func:`colour.models.cie_ucs.UCS_to_uv` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            UCS = np.array(case)
            UCS_to_uv(UCS)


class Testuv_to_UCS(unittest.TestCase):
    """
    Define :func:`colour.models.cie_ucs.uv_to_UCS` definition unit tests
    methods.
    """

    def test_uv_to_UCS(self):
        """Test :func:`colour.models.cie_ucs.uv_to_UCS` definition."""

        np.testing.assert_almost_equal(
            uv_to_UCS(np.array([0.37720213, 0.33413508])),
            np.array([1.12889114, 1.00000000, 0.86391046]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            uv_to_UCS(np.array([0.14536327, 0.35328046])),
            np.array([0.41146705, 1.00000000, 1.41914520]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            uv_to_UCS(np.array([0.16953602, 0.20026156])),
            np.array([0.84657295, 1.00000000, 3.14689659]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            uv_to_UCS(np.array([0.37720213, 0.33413508]), V=0.18),
            np.array([0.20320040, 0.18000000, 0.15550388]),
            decimal=7,
        )

    def test_n_dimensional_uv_to_UCS(self):
        """
        Test :func:`colour.models.cie_ucs.uv_to_UCS` definition n-dimensional
        support.
        """

        uv = np.array([0.37720213, 0.33413508])
        UCS = uv_to_UCS(uv)

        uv = np.tile(uv, (6, 1))
        UCS = np.tile(UCS, (6, 1))
        np.testing.assert_almost_equal(uv_to_UCS(uv), UCS, decimal=7)

        uv = np.reshape(uv, (2, 3, 2))
        UCS = np.reshape(UCS, (2, 3, 3))
        np.testing.assert_almost_equal(uv_to_UCS(uv), UCS, decimal=7)

    def test_domain_range_scale_uv_to_UCS(self):
        """
        Test :func:`colour.models.cie_ucs.uv_to_UCS` definition domain and
        range scale support.
        """

        uv = np.array([0.37720213, 0.33413508])
        V = 1
        UCS = uv_to_UCS(uv, V)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    uv_to_UCS(uv, V * factor), UCS * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_uv_to_UCS(self):
        """Test :func:`colour.models.cie_ucs.uv_to_UCS` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=2))
        for case in cases:
            uv = np.array(case)
            uv_to_UCS(uv)


class TestUCS_uv_to_xy(unittest.TestCase):
    """
    Define :func:`colour.models.cie_ucs.UCS_uv_to_xy` definition unit tests
    methods.
    """

    def test_UCS_uv_to_xy(self):
        """Test :func:`colour.models.cie_ucs.UCS_uv_to_xy` definition."""

        np.testing.assert_almost_equal(
            UCS_uv_to_xy(np.array([0.37720213, 0.33413508])),
            np.array([0.54369555, 0.32107941]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            UCS_uv_to_xy(np.array([0.14536327, 0.35328046])),
            np.array([0.29777734, 0.48246445]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            UCS_uv_to_xy(np.array([0.16953602, 0.20026156])),
            np.array([0.18582823, 0.14633764]),
            decimal=7,
        )

    def test_n_dimensional_UCS_uv_to_xy(self):
        """
        Test :func:`colour.models.cie_ucs.UCS_uv_to_xy` definition
        n-dimensional arrays support.
        """

        uv = np.array([0.37720213, 0.33413508])
        xy = UCS_uv_to_xy(uv)

        uv = np.tile(uv, (6, 1))
        xy = np.tile(xy, (6, 1))
        np.testing.assert_almost_equal(UCS_uv_to_xy(uv), xy, decimal=7)

        uv = np.reshape(uv, (2, 3, 2))
        xy = np.reshape(xy, (2, 3, 2))
        np.testing.assert_almost_equal(UCS_uv_to_xy(uv), xy, decimal=7)

    @ignore_numpy_errors
    def test_nan_UCS_uv_to_xy(self):
        """
        Test :func:`colour.models.cie_ucs.UCS_uv_to_xy` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=2))
        for case in cases:
            uv = np.array(case)
            UCS_uv_to_xy(uv)


class TestXy_to_UCS_uv(unittest.TestCase):
    """
    Define :func:`colour.models.cie_ucs.xy_to_UCS_uv` definition unit tests
    methods.
    """

    def test_xy_to_UCS_uv(self):
        """Test :func:`colour.models.cie_ucs.xy_to_UCS_uv` definition."""

        np.testing.assert_almost_equal(
            xy_to_UCS_uv(np.array([0.54369555, 0.32107941])),
            np.array([0.37720213, 0.33413508]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            xy_to_UCS_uv(np.array([0.29777734, 0.48246445])),
            np.array([0.14536327, 0.35328046]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            xy_to_UCS_uv(np.array([0.18582823, 0.14633764])),
            np.array([0.16953602, 0.20026156]),
            decimal=7,
        )

    def test_n_dimensional_xy_to_UCS_uv(self):
        """
        Test :func:`colour.models.cie_ucs.xy_to_UCS_uv` definition
        n-dimensional arrays support.
        """

        xy = np.array([0.54369555, 0.32107941])
        uv = xy_to_UCS_uv(xy)

        xy = np.tile(xy, (6, 1))
        uv = np.tile(uv, (6, 1))
        np.testing.assert_almost_equal(xy_to_UCS_uv(xy), uv, decimal=7)

        xy = np.reshape(xy, (2, 3, 2))
        uv = np.reshape(uv, (2, 3, 2))
        np.testing.assert_almost_equal(xy_to_UCS_uv(xy), uv, decimal=7)

    @ignore_numpy_errors
    def test_nan_xy_to_UCS_uv(self):
        """
        Test :func:`colour.models.cie_ucs.xy_to_UCS_uv` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=2))
        for case in cases:
            xy = np.array(case)
            xy_to_UCS_uv(xy)


if __name__ == "__main__":
    unittest.main()



from .astm_g_173 import (
    SD_ASTMG173_ETR,
    SD_ASTMG173_GLOBAL_TILT,
    SD_ASTMG173_DIRECT_CIRCUMSOLAR,
)

__all__ = [
    "SD_ASTMG173_ETR",
    "SD_ASTMG173_GLOBAL_TILT",
    "SD_ASTMG173_DIRECT_CIRCUMSOLAR",
]



"""Defines the unit tests for the :mod:`colour.volume.mesh` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.volume import is_within_mesh_volume
from colour.utilities import ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestIsWithinMeshVolume",
]


class TestIsWithinMeshVolume(unittest.TestCase):
    """
    Define :func:`colour.volume.mesh.is_within_mesh_volume` definition unit
    tests methods.
    """

    def setUp(self):
        """Initialise the common tests attributes."""

        self._mesh = np.array(
            [
                [-1.0, -1.0, 1.0],
                [1.0, -1.0, 1.0],
                [1.0, -1.0, -1.0],
                [-1.0, -1.0, -1.0],
                [0.0, 1.0, 0.0],
            ]
        )

    def test_is_within_mesh_volume(self):
        """Test :func:`colour.volume.mesh.is_within_mesh_volume` definition."""

        self.assertTrue(
            is_within_mesh_volume(
                np.array([0.0005, 0.0031, 0.0010]), self._mesh
            )
        )

        self.assertFalse(
            is_within_mesh_volume(
                np.array([0.3205, 0.4131, 0.5100]), self._mesh
            )
        )

        self.assertTrue(
            is_within_mesh_volume(
                np.array([0.0025, 0.0088, 0.0340]), self._mesh
            )
        )

        self.assertFalse(
            is_within_mesh_volume(
                np.array([0.4325, 0.3788, 0.1034]), self._mesh
            )
        )

    def test_n_dimensional_is_within_mesh_volume(self):
        """
        Test :func:`colour.volume.mesh.is_within_mesh_volume` definition
        n-dimensional arrays support.
        """

        a = np.array([0.0005, 0.0031, 0.0010])
        b = is_within_mesh_volume(a, self._mesh)

        a = np.tile(a, (6, 1))
        b = np.tile(b, 6)
        np.testing.assert_almost_equal(is_within_mesh_volume(a, self._mesh), b)

        a = np.reshape(a, (2, 3, 3))
        b = np.reshape(b, (2, 3))
        np.testing.assert_almost_equal(is_within_mesh_volume(a, self._mesh), b)

    @ignore_numpy_errors
    def test_nan_is_within_mesh_volume(self):
        """
        Test :func:`colour.volume.mesh.is_within_mesh_volume` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            is_within_mesh_volume(case, self._mesh)


if __name__ == "__main__":
    unittest.main()



"""
Krystek (1985) Correlated Colour Temperature
============================================

Defines the *Krystek (1985)* correlated colour temperature :math:`T_{cp}`
computations objects:

-   :func:`colour.temperature.uv_to_CCT_Krystek1985`: Correlated colour
    temperature :math:`T_{cp}` computation of given *CIE UCS* colourspace *uv*
    chromaticity coordinates using *Krystek (1985)* method.
-   :func:`colour.temperature.CCT_to_uv_Krystek1985`: *CIE UCS* colourspace
    *uv* chromaticity coordinates computation of given correlated colour
    temperature :math:`T_{cp}` using *Krystek (1985)* method.

References
----------
-   :cite:`Krystek1985b` : Krystek, M. (1985). An algorithm to calculate
    correlated colour temperature. Color Research & Application, 10(1), 38-40.
    doi:10.1002/col.5080100109
"""

from __future__ import annotations

import numpy as np
from scipy.optimize import minimize

from colour.hints import (
    ArrayLike,
    Dict,
    FloatingOrArrayLike,
    FloatingOrNDArray,
    NDArray,
    Optional,
)
from colour.utilities import as_float_array, as_float, tstack

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "uv_to_CCT_Krystek1985",
    "CCT_to_uv_Krystek1985",
]


def uv_to_CCT_Krystek1985(
    uv: ArrayLike, optimisation_kwargs: Optional[Dict] = None
) -> FloatingOrNDArray:
    """
    Return the correlated colour temperature :math:`T_{cp}` from given
    *CIE UCS* colourspace *uv* chromaticity coordinates using *Krystek (1985)*
    method.

    Parameters
    ----------
    uv
         *CIE UCS* colourspace *uv* chromaticity coordinates.
    optimisation_kwargs
        Parameters for :func:`scipy.optimize.minimize` definition.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Correlated colour temperature :math:`T_{cp}`.

    Warnings
    --------
    *Krystek (1985)* does not give an analytical inverse transformation to
    compute the correlated colour temperature :math:`T_{cp}` from given
    *CIE UCS* colourspace *uv* chromaticity coordinates, the current
    implementation relies on optimization using :func:`scipy.optimize.minimize`
    definition and thus has reduced precision and poor performance.

    Notes
    -----
    -   *Krystek (1985)* method computations are valid for correlated colour
        temperature :math:`T_{cp}` normalised to domain [1000, 15000].

    References
    ----------
    :cite:`Krystek1985b`

    Examples
    --------
    >>> uv_to_CCT_Krystek1985(np.array([0.20047203, 0.31029290]))
    ... # doctest: +ELLIPSIS
    6504.3894290...
    """

    uv = as_float_array(uv)
    shape = uv.shape
    uv = np.atleast_1d(uv.reshape([-1, 2]))

    def objective_function(
        CCT: FloatingOrArrayLike, uv: ArrayLike
    ) -> FloatingOrNDArray:
        """Objective function."""

        objective = np.linalg.norm(CCT_to_uv_Krystek1985(CCT) - uv)

        return as_float(objective)

    optimisation_settings = {
        "method": "Nelder-Mead",
        "options": {
            "fatol": 1e-10,
        },
    }
    if optimisation_kwargs is not None:
        optimisation_settings.update(optimisation_kwargs)

    CCT = as_float_array(
        [
            minimize(
                objective_function,
                x0=6500,
                args=(uv_i,),
                **optimisation_settings,
            ).x
            for uv_i in as_float_array(uv)
        ]
    )

    return as_float(np.reshape(CCT, shape[:-1]))


def CCT_to_uv_Krystek1985(CCT: FloatingOrArrayLike) -> NDArray:
    """
    Return the *CIE UCS* colourspace *uv* chromaticity coordinates from given
    correlated colour temperature :math:`T_{cp}` using *Krystek (1985)* method.

    Parameters
    ----------
    CCT
        Correlated colour temperature :math:`T_{cp}`.

    Returns
    -------
    :class:`numpy.ndarray`
        *CIE UCS* colourspace *uv* chromaticity coordinates.

    Notes
    -----
    -   *Krystek (1985)* method computations are valid for correlated colour
        temperature :math:`T_{cp}` normalised to domain [1000, 15000].

    References
    ----------
    :cite:`Krystek1985b`

    Examples
    --------
    >>> CCT_to_uv_Krystek1985(6504.38938305)  # doctest: +ELLIPSIS
    array([ 0.2004720...,  0.3102929...])
    """

    T = as_float_array(CCT)

    T_2 = T**2

    u = (
        0.860117757 + 1.54118254 * 10**-4 * T + 1.28641212 * 10**-7 * T_2
    ) / (1 + 8.42420235 * 10**-4 * T + 7.08145163 * 10**-7 * T_2)
    v = (
        0.317398726 + 4.22806245 * 10**-5 * T + 4.20481691 * 10**-8 * T_2
    ) / (1 - 2.89741816 * 10**-5 * T + 1.61456053 * 10**-7 * T_2)

    return tstack([u, v])



"""Showcases *CMCCAT2000* chromatic adaptation model computations."""

import numpy as np

import colour
from colour.utilities import message_box

message_box('"CMCCAT200" Chromatic Adaptation Model Computations')

XYZ = np.array([0.2248, 0.2274, 0.0854])
XYZ_w = np.array([1.1115, 1.0000, 0.3520])
XYZ_wr = np.array([0.9481, 1.0000, 1.0730])
L_A1 = 200
L_A2 = 200
message_box(
    f'Computing chromatic adaptation using "CMCCAT200" forward chromatic '
    f"adaptation model.\n\n"
    f'\t"XYZ":\n\t\t{XYZ}\n'
    f'\t"XYZ_w":\n\t\t{XYZ_w}\n'
    f'\t"XYZ_wr":\n\t\t{XYZ_wr}\n'
    f'\t"L_A1":\n\t\t{L_A1}\n'
    f'\t"L_A2":\n\t\t{L_A2}'
)
print(
    colour.chromatic_adaptation(
        XYZ, XYZ_w, XYZ_wr, method="CMCCAT2000", L_A1=L_A1, L_A2=L_A2
    )
)
print(
    colour.adaptation.chromatic_adaptation_CMCCAT2000(
        XYZ * 100, XYZ_w, XYZ_wr, L_A1, L_A2
    )
    / 100
)

print("\n")

XYZ_c = np.array([0.19526983, 0.23068340, 0.24971752])
message_box(
    f'Computing chromatic adaptation using "CMCCAT200" inverse chromatic '
    f"adaptation model.\n\n"
    f'\t"XYZ_c": {XYZ_c}\n'
    f'\t"XYZ_w": {XYZ_w}\n'
    f'\t"XYZ_wr": {XYZ_wr}\n'
    f'\t"L_A1": {L_A1}\n'
    f'\t"L_A2": {L_A2}'
)
print(
    colour.chromatic_adaptation(
        XYZ_c,
        XYZ_w,
        XYZ_wr,
        method="CMCCAT2000",
        L_A1=L_A1,
        L_A2=L_A2,
        direction="Inverse",
    )
)
print(
    colour.adaptation.chromatic_adaptation_CMCCAT2000(
        XYZ_c * 100, XYZ_w, XYZ_wr, L_A1, L_A2, direction="Inverse"
    )
    / 100
)



"""Showcases hexadecimal computations."""

import numpy as np

import colour.notation.hexadecimal
from colour.utilities import message_box

message_box("Hexadecimal Computations")

RGB = np.array([0.45620519, 0.03081071, 0.04091952])
message_box(
    f'Converting to the "hexadecimal" representation from given "RGB"'
    f"colourspace values:\n\n\t{RGB}"
)
print(colour.notation.hexadecimal.RGB_to_HEX(RGB))

print("\n")

hex_triplet = "#74070a"
message_box(
    f'Converting to the "RGB" colourspace from given "hexadecimal" '
    f"representation:\n\n\t{hex_triplet}"
)
print(colour.notation.hexadecimal.HEX_to_RGB(hex_triplet))



"""Showcases colour component transfer functions (CCTF) relates computations."""

import colour
from colour.utilities import message_box

message_box("Colour Component Transfer Functions (CCTF) Computations")

C = 18 / 100

message_box(
    f'Encoding to video component signal value using "BT.709" OETF and given '
    f"linear-light value:\n\n\t{C}"
)
print(colour.oetf(C, function="ITU-R BT.709"))
print(colour.models.oetf_BT709(C))

print("\n")

N = 0.40900773
message_box(
    f'Decoding to linear-light value using "BT.1886" EOTF and given video '
    f"component signal value:\n\n\t{N}"
)
print(colour.eotf(N, function="ITU-R BT.1886"))
print(colour.models.eotf_BT1886(N))

print("\n")

message_box(f'Encoding to "Cineon" using given linear-light value:\n\n\t{C}')
print(colour.log_encoding(C, function="Cineon"))
print(colour.models.log_encoding_Cineon(C))

print("\n")

N = 0.45731961
message_box(
    f'Decoding to linear-light using given "Cineon" code value:\n\n\t{N}'
)
print(colour.log_decoding(N, function="Cineon"))
print(colour.models.log_decoding_Cineon(N))

print("\n")

message_box(f'Encoding to "PLog" using given linear-light value:\n\n\t{C}')
print(colour.log_encoding(C, function="PLog"))
print(colour.models.log_encoding_PivotedLog(C))

print("\n")

N = 0.43499511
message_box(
    f'Decoding to linear-light value using given "PLog" code value:\n\n\t{N}'
)
print(colour.log_decoding(N, function="PLog"))
print(colour.models.log_decoding_PivotedLog(N))

print("\n")

message_box(
    f"Encoding to video component signal value using a pure gamma function "
    f"and given linear-light value:\n\n\t{C}"
)
print(colour.gamma_function(C, 1 / 2.2))

print("\n")

N = 0.45865645
message_box(
    f"Decoding to linear-light value using a pure gamma function and given "
    f"video component signal value:\n\n\t{N}"
)
print(colour.gamma_function(N, 2.2))



"""Defines the unit tests for the :mod:`colour.models.rgb.cmyk` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.models.rgb.cmyk import (
    RGB_to_CMY,
    CMY_to_RGB,
    CMY_to_CMYK,
    CMYK_to_CMY,
)
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestRGB_to_CMY",
    "TestCMY_to_RGB",
    "TestCMY_to_CMYK",
    "TestCMYK_to_CMY",
]


class TestRGB_to_CMY(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.cmyk.RGB_to_CMY` definition unit tests
    methods.
    """

    def test_RGB_to_CMY(self):
        """Test :func:`colour.models.rgb.cmyk.RGB_to_CMY` definition."""

        np.testing.assert_almost_equal(
            RGB_to_CMY(np.array([0.45620519, 0.03081071, 0.04091952])),
            np.array([0.54379481, 0.96918929, 0.95908048]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            RGB_to_CMY(np.array([0.00000000, 0.00000000, 0.00000000])),
            np.array([1.00000000, 1.00000000, 1.00000000]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            RGB_to_CMY(np.array([1.00000000, 1.00000000, 1.00000000])),
            np.array([0.00000000, 0.00000000, 0.00000000]),
            decimal=7,
        )

    def test_n_dimensional_RGB_to_CMY(self):
        """
        Test :func:`colour.models.rgb.cmyk.RGB_to_CMY` definition
        n-dimensional arrays support.
        """

        RGB = np.array([0.45620519, 0.03081071, 0.04091952])
        CMY = RGB_to_CMY(RGB)

        RGB = np.tile(RGB, (6, 1))
        CMY = np.tile(CMY, (6, 1))
        np.testing.assert_almost_equal(RGB_to_CMY(RGB), CMY, decimal=7)

        RGB = np.reshape(RGB, (2, 3, 3))
        CMY = np.reshape(CMY, (2, 3, 3))
        np.testing.assert_almost_equal(RGB_to_CMY(RGB), CMY, decimal=7)

    def test_domain_range_scale_RGB_to_CMY(self):
        """
        Test :func:`colour.models.rgb.cmyk.RGB_to_CMY` definition domain and
        range scale support.
        """

        RGB = np.array([0.45620519, 0.03081071, 0.04091952])
        CMY = RGB_to_CMY(RGB)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    RGB_to_CMY(RGB * factor), CMY * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_RGB_to_CMY(self):
        """
        Test :func:`colour.models.rgb.cmyk.RGB_to_CMY` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            RGB = np.array(case)
            RGB_to_CMY(RGB)


class TestCMY_to_RGB(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.cmyk.CMY_to_RGB` definition unit tests
    methods.
    """

    def test_CMY_to_RGB(self):
        """Test :func:`colour.models.rgb.cmyk.CMY_to_RGB` definition."""

        np.testing.assert_almost_equal(
            CMY_to_RGB(np.array([0.54379481, 0.96918929, 0.95908048])),
            np.array([0.45620519, 0.03081071, 0.04091952]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            CMY_to_RGB(np.array([1.00000000, 1.00000000, 1.00000000])),
            np.array([0.00000000, 0.00000000, 0.00000000]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            CMY_to_RGB(np.array([0.00000000, 0.00000000, 0.00000000])),
            np.array([1.00000000, 1.00000000, 1.00000000]),
            decimal=7,
        )

    def test_n_dimensional_CMY_to_RGB(self):
        """
        Test :func:`colour.models.rgb.cmyk.CMY_to_RGB` definition
        n-dimensional arrays support.
        """

        CMY = np.array([0.54379481, 0.96918929, 0.95908048])
        RGB = CMY_to_RGB(CMY)

        CMY = np.tile(CMY, (6, 1))
        RGB = np.tile(RGB, (6, 1))
        np.testing.assert_almost_equal(CMY_to_RGB(CMY), RGB, decimal=7)

        CMY = np.reshape(CMY, (2, 3, 3))
        RGB = np.reshape(RGB, (2, 3, 3))
        np.testing.assert_almost_equal(CMY_to_RGB(CMY), RGB, decimal=7)

    def test_domain_range_scale_CMY_to_RGB(self):
        """
        Test :func:`colour.models.rgb.cmyk.CMY_to_RGB` definition domain and
        range scale support.
        """

        CMY = np.array([0.54379481, 0.96918929, 0.95908048])
        RGB = CMY_to_RGB(CMY)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    CMY_to_RGB(CMY * factor), RGB * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_CMY_to_RGB(self):
        """Test :func:`colour.models.rgb.cmyk.CMY_to_RGB` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            CMY = np.array(case)
            CMY_to_RGB(CMY)


class TestCMY_to_CMYK(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.cmyk.CMY_to_CMYK` definition unit tests
    methods.
    """

    def test_CMY_to_CMYK(self):
        """Test :func:`colour.models.rgb.cmyk.CMY_to_CMYK` definition."""

        np.testing.assert_almost_equal(
            CMY_to_CMYK(np.array([0.54379481, 0.96918929, 0.95908048])),
            np.array([0.00000000, 0.93246304, 0.91030457, 0.54379481]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            CMY_to_CMYK(np.array([0.15000000, 1.00000000, 1.00000000])),
            np.array([0.00000000, 1.00000000, 1.00000000, 0.15000000]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            CMY_to_CMYK(np.array([0.15000000, 0.00000000, 0.00000000])),
            np.array([0.15000000, 0.00000000, 0.00000000, 0.00000000]),
            decimal=7,
        )

    def test_n_dimensional_CMY_to_CMYK(self):
        """
        Test :func:`colour.models.rgb.cmyk.CMY_to_CMYK` definition
        n-dimensional arrays support.
        """

        CMY = np.array([0.54379481, 0.96918929, 0.95908048])
        CMYK = CMY_to_CMYK(CMY)

        CMY = np.tile(CMY, (6, 1))
        CMYK = np.tile(CMYK, (6, 1))
        np.testing.assert_almost_equal(CMY_to_CMYK(CMY), CMYK, decimal=7)

        CMY = np.reshape(CMY, (2, 3, 3))
        CMYK = np.reshape(CMYK, (2, 3, 4))
        np.testing.assert_almost_equal(CMY_to_CMYK(CMY), CMYK, decimal=7)

    def test_domain_range_scale_CMY_to_CMYK(self):
        """
        Test :func:`colour.models.rgb.cmyk.CMY_to_CMYK` definition domain and
        range scale support.
        """

        CMY = np.array([0.54379481, 0.96918929, 0.95908048])
        CMYK = CMY_to_CMYK(CMY)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    CMY_to_CMYK(CMY * factor), CMYK * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_CMY_to_CMYK(self):
        """
        Test :func:`colour.models.rgb.cmyk.CMY_to_CMYK` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            CMY = np.array(case)
            CMY_to_CMYK(CMY)


class TestCMYK_to_CMY(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.cmyk.CMYK_to_CMY` definition unit tests
    methods.
    """

    def test_CMYK_to_CMY(self):
        """Test :func:`colour.models.rgb.cmyk.CMYK_to_CMY` definition."""

        np.testing.assert_almost_equal(
            CMYK_to_CMY(
                np.array([0.00000000, 0.93246304, 0.91030457, 0.54379481])
            ),
            np.array([0.54379481, 0.96918929, 0.95908048]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            CMYK_to_CMY(
                np.array([0.00000000, 1.00000000, 1.00000000, 0.15000000])
            ),
            np.array([0.15000000, 1.00000000, 1.00000000]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            CMYK_to_CMY(
                np.array([0.15000000, 0.00000000, 0.00000000, 0.00000000])
            ),
            np.array([0.15000000, 0.00000000, 0.00000000]),
            decimal=7,
        )

    def test_n_dimensional_CMYK_to_CMY(self):
        """
        Test :func:`colour.models.rgb.cmyk.CMYK_to_CMY` definition
        n-dimensional arrays support.
        """

        CMYK = np.array([0.00000000, 0.93246304, 0.91030457, 0.54379481])
        CMY = CMYK_to_CMY(CMYK)

        CMYK = np.tile(CMYK, (6, 1))
        CMY = np.tile(CMY, (6, 1))
        np.testing.assert_almost_equal(CMYK_to_CMY(CMYK), CMY, decimal=7)

        CMYK = np.reshape(CMYK, (2, 3, 4))
        CMY = np.reshape(CMY, (2, 3, 3))
        np.testing.assert_almost_equal(CMYK_to_CMY(CMYK), CMY, decimal=7)

    def test_domain_range_scale_CMYK_to_CMY(self):
        """
        Test :func:`colour.models.rgb.cmyk.CMYK_to_CMY` definition domain and
        range scale support.
        """

        CMYK = np.array([0.00000000, 0.93246304, 0.91030457, 0.54379481])
        CMY = CMYK_to_CMY(CMYK)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    CMYK_to_CMY(CMYK * factor), CMY * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_CMYK_to_CMY(self):
        """
        Test :func:`colour.models.rgb.cmyk.CMYK_to_CMY` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=4))
        for case in cases:
            CMYK = np.array(case)
            CMYK_to_CMY(CMYK)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""Showcases Machado (2009) simulation of colour vision deficiency."""

import numpy as np

import colour
from colour.utilities.verbose import message_box

message_box("Simulation of CVD - Machado (2009)")

M_a = colour.matrix_anomalous_trichromacy_Machado2009(
    colour.MSDS_CMFS["Stockman & Sharpe 2 Degree Cone Fundamentals"],
    colour.MSDS_DISPLAY_PRIMARIES["Typical CRT Brainard 1997"],
    np.array([10, 0, 0]),
)
message_box(
    f'Computing a "Protanomaly" matrix using '
    f'"Stockman & Sharpe 2 Degree Cone Fundamentals" and '
    f'"Typical CRT Brainard 1997" "RGB" display primaries for a 10nm shift:\n\n{M_a}'
)

print("\n")

M_a = colour.matrix_cvd_Machado2009("Protanomaly", 0.5)
message_box(
    f'Retrieving a "Protanomaly" pre-computed matrix for a 50% severity:\n\n{M_a}'
)

print("\n")

M_a = colour.matrix_anomalous_trichromacy_Machado2009(
    colour.MSDS_CMFS["Stockman & Sharpe 2 Degree Cone Fundamentals"],
    colour.MSDS_DISPLAY_PRIMARIES["Typical CRT Brainard 1997"],
    np.array([0, 10, 0]),
)
message_box(
    f'Computing a "Deuteranomaly" matrix using '
    f'"Stockman & Sharpe 2 Degree Cone Fundamentals" and '
    f'"Typical CRT Brainard 1997" "RGB" display primaries for a 10nm shift:\n\n{M_a}'
)

print("\n")

M_a = colour.matrix_cvd_Machado2009("Deuteranomaly", 0.5)
message_box(
    f'Retrieving a "Deuteranomaly" pre-computed matrix for a 50% severity:\n\n{M_a}'
)

print("\n")

M_a = colour.matrix_anomalous_trichromacy_Machado2009(
    colour.MSDS_CMFS["Stockman & Sharpe 2 Degree Cone Fundamentals"],
    colour.MSDS_DISPLAY_PRIMARIES["Typical CRT Brainard 1997"],
    np.array([0, 0, 27]),
)
message_box(
    f'Computing a "Tritanomaly" matrix using '
    f'"Stockman & Sharpe 2 Degree Cone Fundamentals" and '
    f'"Typical CRT Brainard 1997" "RGB" display primaries for a 27nm shift:\n\n{M_a}'
)

print("\n")

M_a = colour.matrix_cvd_Machado2009("Tritanomaly", 0.5)
message_box(
    f'Retrieving a "Tritanomaly" pre-computed matrix for a 50% severity:\n\n{M_a}'
)



"""
Random Numbers Utilities
========================

Defines the random number generator objects:

-   :func:`colour.algebra.random_triplet_generator`
References
----------
-   :cite:`Laurent2012a` : Laurent. (2012). Reproducibility of python
    pseudo-random numbers across systems and versions? Retrieved January 20,
    2015, from
    http://stackoverflow.com/questions/8786084/\
reproducibility-of-python-pseudo-random-numbers-across-systems-and-versions
"""

from __future__ import annotations

import numpy as np

from colour.hints import ArrayLike, Integer, NDArray
from colour.utilities import as_float_array, tstack

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "RANDOM_STATE",
    "random_triplet_generator",
]

RANDOM_STATE = np.random.RandomState()


def random_triplet_generator(
    size: Integer,
    limits: ArrayLike = np.array([[0, 1], [0, 1], [0, 1]]),
    random_state: np.random.RandomState = RANDOM_STATE,
) -> NDArray:
    """
    Return a generator yielding random triplets.

    Parameters
    ----------
    size
        Generator size.
    limits
        Random values limits on each triplet axis.
    random_state
         Mersenne Twister pseudo-random number generator.

    Returns
    -------
    :class:`numpy.ndarray`
        Random triplet generator.

    Notes
    -----
    -   The test is assuming that :func:`np.random.RandomState` definition
        will return the same sequence no matter which *OS* or *Python* version
        is used. There is however no formal promise about the *prng* sequence
        reproducibility of either *Python* or *Numpy* implementations, see
        :cite:`Laurent2012a`.

    Examples
    --------
    >>> from pprint import pprint
    >>> prng = np.random.RandomState(4)
    >>> random_triplet_generator(10, random_state=prng)
    ... # doctest: +ELLIPSIS
    array([[ 0.9670298...,  0.7793829...,  0.4361466...],
           [ 0.5472322...,  0.1976850...,  0.9489773...],
           [ 0.9726843...,  0.8629932...,  0.7863059...],
           [ 0.7148159...,  0.9834006...,  0.8662893...],
           [ 0.6977288...,  0.1638422...,  0.1731654...],
           [ 0.2160895...,  0.5973339...,  0.0749485...],
           [ 0.9762744...,  0.0089861...,  0.6007427...],
           [ 0.0062302...,  0.3865712...,  0.1679721...],
           [ 0.2529823...,  0.0441600...,  0.7333801...],
           [ 0.4347915...,  0.9566529...,  0.4084438...]])
    """

    limit_x, limit_y, limit_z = as_float_array(limits)

    return tstack(
        [
            random_state.uniform(limit_x[0], limit_x[1], size=size),
            random_state.uniform(limit_y[0], limit_y[1], size=size),
            random_state.uniform(limit_z[0], limit_z[1], size=size),
        ]
    )



"""
Regression
==========

Defines various objects to perform regression:

-   :func:`colour.algebra.least_square_mapping_MoorePenrose`: *Least-squares*
    mapping using *Moore-Penrose* inverse.

References
----------
-   :cite:`Finlayson2015` : Finlayson, G. D., MacKiewicz, M., & Hurlbert, A.
    (2015). Color Correction Using Root-Polynomial Regression. IEEE
    Transactions on Image Processing, 24(5), 1460-1470.
    doi:10.1109/TIP.2015.2405336
"""

from __future__ import annotations

import numpy as np

from colour.hints import ArrayLike, NDArray

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "least_square_mapping_MoorePenrose",
]


def least_square_mapping_MoorePenrose(y: ArrayLike, x: ArrayLike) -> NDArray:
    """
    Compute the *least-squares* mapping from dependent variable :math:`y` to
    independent variable :math:`x` using *Moore-Penrose* inverse.

    Parameters
    ----------
    y
        Dependent and already known :math:`y` variable.
    x
        Independent :math:`x` variable(s) values corresponding with :math:`y`
        variable.

    Returns
    -------
    :class:`numpy.ndarray`
        *Least-squares* mapping.

    References
    ----------
    :cite:`Finlayson2015`

    Examples
    --------
    >>> prng = np.random.RandomState(2)
    >>> y = prng.random_sample((24, 3))
    >>> x = y + (prng.random_sample((24, 3)) - 0.5) * 0.5
    >>> least_square_mapping_MoorePenrose(y, x)  # doctest: +ELLIPSIS
    array([[ 1.0526376...,  0.1378078..., -0.2276339...],
           [ 0.0739584...,  1.0293994..., -0.1060115...],
           [ 0.0572550..., -0.2052633...,  1.1015194...]])
    """

    y = np.atleast_2d(y)
    x = np.atleast_2d(x)

    return np.dot(np.transpose(x), np.linalg.pinv(np.transpose(y)))



"""
Defines the unit tests for the :mod:`colour.algebra.interpolation` module.

References
----------
-   :cite:`Fairman1985b` : Fairman, H. S. (1985). The calculation of weight
    factors for tristimulus integration. Color Research & Application, 10(4),
    199-203. doi:10.1002/col.5080100407
"""

from __future__ import annotations

import numpy as np
import os
import unittest
from itertools import permutations

from colour.algebra.interpolation import vertices_and_relative_coordinates
from colour.hints import NDArray, Tuple
from colour.algebra import (
    kernel_nearest_neighbour,
    kernel_linear,
    kernel_sinc,
    kernel_lanczos,
    kernel_cardinal_spline,
    KernelInterpolator,
    NearestNeighbourInterpolator,
    LinearInterpolator,
    SpragueInterpolator,
    CubicSplineInterpolator,
    PchipInterpolator,
    NullInterpolator,
    lagrange_coefficients,
    table_interpolation_trilinear,
    table_interpolation_tetrahedral,
)
from colour.algebra import random_triplet_generator
from colour.io import read_LUT
from colour.utilities import ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "DATA_POINTS_A",
    "DATA_POINTS_A_LINEAR_INTERPOLATED_10_SAMPLES",
    "DATA_POINTS_A_SPRAGUE_INTERPOLATED_10_SAMPLES",
    "DATA_POINTS_A_CUBIC_SPLINE_INTERPOLATED_X2_SAMPLES",
    "LAGRANGE_COEFFICIENTS_A",
    "LAGRANGE_COEFFICIENTS_B",
    "LUT_TABLE",
    "TestKernelNearestNeighbour",
    "TestKernelLinear",
    "TestKernelSinc",
    "TestKernelLanczos",
    "TestKernelCardinalSpline",
    "TestKernelInterpolator",
    "TestLinearInterpolator",
    "TestSpragueInterpolator",
    "TestCubicSplineInterpolator",
    "TestPchipInterpolator",
    "TestNullInterpolator",
    "TestLagrangeCoefficients",
    "TestVerticesAndRelativeCoordinates",
    "TestTableInterpolationTrilinear",
    "TestTableInterpolationTetrahedral",
]

DATA_POINTS_A: Tuple = (
    9.3700,
    12.3200,
    12.4600,
    9.5100,
    5.9200,
    4.3300,
    4.2900,
    3.8800,
    4.5100,
    10.9200,
    27.5000,
    49.6700,
    69.5900,
    81.7300,
    88.1900,
    86.0500,
)

DATA_POINTS_A_LINEAR_INTERPOLATED_10_SAMPLES: Tuple = (
    9.370,
    9.665,
    9.960,
    10.255,
    10.550,
    10.845,
    11.140,
    11.435,
    11.730,
    12.025,
    12.320,
    12.334,
    12.348,
    12.362,
    12.376,
    12.390,
    12.404,
    12.418,
    12.432,
    12.446,
    12.460,
    12.165,
    11.870,
    11.575,
    11.280,
    10.985,
    10.690,
    10.395,
    10.100,
    9.805,
    9.510,
    9.151,
    8.792,
    8.433,
    8.074,
    7.715,
    7.356,
    6.997,
    6.638,
    6.279,
    5.920,
    5.761,
    5.602,
    5.443,
    5.284,
    5.125,
    4.966,
    4.807,
    4.648,
    4.489,
    4.330,
    4.326,
    4.322,
    4.318,
    4.314,
    4.310,
    4.306,
    4.302,
    4.298,
    4.294,
    4.290,
    4.249,
    4.208,
    4.167,
    4.126,
    4.085,
    4.044,
    4.003,
    3.962,
    3.921,
    3.880,
    3.943,
    4.006,
    4.069,
    4.132,
    4.195,
    4.258,
    4.321,
    4.384,
    4.447,
    4.510,
    5.151,
    5.792,
    6.433,
    7.074,
    7.715,
    8.356,
    8.997,
    9.638,
    10.279,
    10.920,
    12.578,
    14.236,
    15.894,
    17.552,
    19.210,
    20.868,
    22.526,
    24.184,
    25.842,
    27.500,
    29.717,
    31.934,
    34.151,
    36.368,
    38.585,
    40.802,
    43.019,
    45.236,
    47.453,
    49.670,
    51.662,
    53.654,
    55.646,
    57.638,
    59.630,
    61.622,
    63.614,
    65.606,
    67.598,
    69.590,
    70.804,
    72.018,
    73.232,
    74.446,
    75.660,
    76.874,
    78.088,
    79.302,
    80.516,
    81.730,
    82.376,
    83.022,
    83.668,
    84.314,
    84.960,
    85.606,
    86.252,
    86.898,
    87.544,
    88.190,
    87.976,
    87.762,
    87.548,
    87.334,
    87.120,
    86.906,
    86.692,
    86.478,
    86.264,
    86.050,
)

DATA_POINTS_A_SPRAGUE_INTERPOLATED_10_SAMPLES: Tuple = (
    9.37000000,
    9.72075073,
    10.06936191,
    10.41147570,
    10.74302270,
    11.06022653,
    11.35960827,
    11.63799100,
    11.89250427,
    12.12058860,
    12.32000000,
    12.48873542,
    12.62489669,
    12.72706530,
    12.79433478,
    12.82623598,
    12.82266243,
    12.78379557,
    12.71003009,
    12.60189921,
    12.46000000,
    12.28440225,
    12.07404800,
    11.82976500,
    11.55443200,
    11.25234375,
    10.92857600,
    10.58835050,
    10.23640000,
    9.87633325,
    9.51000000,
    9.13692962,
    8.75620800,
    8.36954763,
    7.98097600,
    7.59601562,
    7.22086400,
    6.86157362,
    6.52323200,
    6.20914162,
    5.92000000,
    5.65460200,
    5.41449600,
    5.20073875,
    5.01294400,
    4.84968750,
    4.70891200,
    4.58833225,
    4.48584000,
    4.39990900,
    4.33000000,
    4.27757887,
    4.24595200,
    4.23497388,
    4.24099200,
    4.25804688,
    4.27907200,
    4.29709387,
    4.30643200,
    4.30389887,
    4.29000000,
    4.26848387,
    4.24043200,
    4.20608887,
    4.16603200,
    4.12117188,
    4.07275200,
    4.02234887,
    3.97187200,
    3.92356387,
    3.88000000,
    3.84319188,
    3.81318400,
    3.79258487,
    3.78691200,
    3.80367187,
    3.85144000,
    3.93894087,
    4.07412800,
    4.26326387,
    4.51000000,
    4.81362075,
    5.17028800,
    5.58225150,
    6.05776000,
    6.60890625,
    7.24947200,
    7.99277300,
    8.84950400,
    9.82558375,
    10.92000000,
    12.12700944,
    13.44892800,
    14.88581406,
    16.43283200,
    18.08167969,
    19.82201600,
    21.64288831,
    23.53416000,
    25.48793794,
    27.50000000,
    29.57061744,
    31.69964800,
    33.88185481,
    36.10777600,
    38.36511719,
    40.64014400,
    42.91907456,
    45.18947200,
    47.44163694,
    49.67000000,
    51.87389638,
    54.05273600,
    56.20157688,
    58.31198400,
    60.37335938,
    62.37427200,
    64.30378787,
    66.15280000,
    67.91535838,
    69.59000000,
    71.17616669,
    72.66283200,
    74.04610481,
    75.33171200,
    76.53183594,
    77.66195200,
    78.73766606,
    79.77155200,
    80.76998919,
    81.73000000,
    82.64375688,
    83.51935227,
    84.35919976,
    85.15567334,
    85.89451368,
    86.55823441,
    87.12952842,
    87.59467414,
    87.94694187,
    88.19000000,
    88.33345751,
    88.37111372,
    88.30221714,
    88.13600972,
    87.88846516,
    87.57902706,
    87.22734720,
    86.85002373,
    86.45733945,
    86.05000000,
)

DATA_POINTS_A_CUBIC_SPLINE_INTERPOLATED_X2_SAMPLES: Tuple = (
    9.37000000,
    11.08838189,
    12.26359953,
    12.78808025,
    12.55425139,
    11.50391691,
    9.87473603,
    8.01707329,
    6.30369624,
    5.08664365,
    4.43550284,
    4.25438019,
    4.29206798,
    4.21753374,
    3.98875865,
    3.79691327,
    4.02534907,
    5.23223510,
    8.08816250,
    13.36306794,
    21.19519815,
    30.89350026,
    41.64531611,
    52.53540869,
    62.65180882,
    71.10713687,
    77.46889540,
    82.31355134,
    86.05208477,
    88.28078752,
    88.45998434,
    86.05000000,
)

LAGRANGE_COEFFICIENTS_A: NDArray = np.array(
    [
        [0.92625, 0.09750, -0.02375],
        [0.85500, 0.19000, -0.04500],
        [0.78625, 0.27750, -0.06375],
        [0.72000, 0.36000, -0.08000],
        [0.65625, 0.43750, -0.09375],
        [0.59500, 0.51000, -0.10500],
        [0.53625, 0.57750, -0.11375],
        [0.48000, 0.64000, -0.12000],
        [0.42625, 0.69750, -0.12375],
        [0.37500, 0.75000, -0.12500],
        [0.32625, 0.79750, -0.12375],
        [0.28000, 0.84000, -0.12000],
        [0.23625, 0.87750, -0.11375],
        [0.19500, 0.91000, -0.10500],
        [0.15625, 0.93750, -0.09375],
        [0.12000, 0.96000, -0.08000],
        [0.08625, 0.97750, -0.06375],
        [0.05500, 0.99000, -0.04500],
        [0.02625, 0.99750, -0.02375],
    ]
)

LAGRANGE_COEFFICIENTS_B: NDArray = np.array(
    [
        [-0.0154375, 0.9725625, 0.0511875, -0.0083125],
        [-0.0285000, 0.9405000, 0.1045000, -0.0165000],
        [-0.0393125, 0.9041875, 0.1595625, -0.0244375],
        [-0.0480000, 0.8640000, 0.2160000, -0.0320000],
        [-0.0546875, 0.8203125, 0.2734375, -0.0390625],
        [-0.0595000, 0.7735000, 0.3315000, -0.0455000],
        [-0.0625625, 0.7239375, 0.3898125, -0.0511875],
        [-0.0640000, 0.6720000, 0.4480000, -0.0560000],
        [-0.0639375, 0.6180625, 0.5056875, -0.0598125],
        [-0.0625000, 0.5625000, 0.5625000, -0.0625000],
        [-0.0598125, 0.5056875, 0.6180625, -0.0639375],
        [-0.0560000, 0.4480000, 0.6720000, -0.0640000],
        [-0.0511875, 0.3898125, 0.7239375, -0.0625625],
        [-0.0455000, 0.3315000, 0.7735000, -0.0595000],
        [-0.0390625, 0.2734375, 0.8203125, -0.0546875],
        [-0.0320000, 0.2160000, 0.8640000, -0.0480000],
        [-0.0244375, 0.1595625, 0.9041875, -0.0393125],
        [-0.0165000, 0.1045000, 0.9405000, -0.0285000],
        [-0.0083125, 0.0511875, 0.9725625, -0.0154375],
    ]
)

LUT_TABLE: NDArray = read_LUT(
    os.path.join(
        os.path.dirname(__file__),
        "..",
        "..",
        "io",
        "luts",
        "tests",
        "resources",
        "iridas_cube",
        "Colour_Correct.cube",
    )
).table  # type: ignore[union-attr]


class TestKernelNearestNeighbour(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.kernel_nearest_neighbour`
    definition unit tests methods.
    """

    def test_kernel_nearest(self):
        """
        Test :func:`colour.algebra.interpolation.kernel_nearest_neighbour`
        definition.
        """

        np.testing.assert_almost_equal(
            kernel_nearest_neighbour(np.linspace(-5, 5, 25)),
            np.array(
                [
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    1,
                    1,
                    1,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                ]
            ),
            decimal=7,
        )


class TestKernelLinear(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.kernel_linear` definition
    unit tests methods.
    """

    def test_kernel_linear(self):
        """Test :func:`colour.algebra.interpolation.kernel_linear` definition."""

        np.testing.assert_almost_equal(
            kernel_linear(np.linspace(-5, 5, 25)),
            np.array(
                [
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.16666667,
                    0.58333333,
                    1.00000000,
                    0.58333333,
                    0.16666667,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                ]
            ),
            decimal=7,
        )


class TestKernelSinc(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.kernel_sinc` definition
    unit tests methods.
    """

    def test_kernel_sinc(self):
        """Test :func:`colour.algebra.interpolation.kernel_sinc` definition."""

        np.testing.assert_almost_equal(
            kernel_sinc(np.linspace(-5, 5, 25)),
            np.array(
                [
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.02824617,
                    0.12732395,
                    0.03954464,
                    -0.16539867,
                    -0.18006326,
                    0.19098593,
                    0.73791298,
                    1.00000000,
                    0.73791298,
                    0.19098593,
                    -0.18006326,
                    -0.16539867,
                    0.03954464,
                    0.12732395,
                    0.02824617,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                ]
            ),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            kernel_sinc(np.linspace(-5, 5, 25), 1),
            np.array(
                [
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.19098593,
                    0.73791298,
                    1.00000000,
                    0.73791298,
                    0.19098593,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                ]
            ),
            decimal=7,
        )


class TestKernelLanczos(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.kernel_lanczos` definition
    unit tests methods.
    """

    def test_kernel_lanczos(self):
        """Test :func:`colour.algebra.interpolation.kernel_lanczos` definition."""

        np.testing.assert_almost_equal(
            kernel_lanczos(np.linspace(-5, 5, 25)),
            np.array(
                [
                    0.00000000e00,
                    0.00000000e00,
                    0.00000000e00,
                    0.00000000e00,
                    0.00000000e00,
                    8.06009483e-04,
                    2.43170841e-02,
                    1.48478897e-02,
                    -9.33267411e-02,
                    -1.32871018e-01,
                    1.67651704e-01,
                    7.14720157e-01,
                    1.00000000e00,
                    7.14720157e-01,
                    1.67651704e-01,
                    -1.32871018e-01,
                    -9.33267411e-02,
                    1.48478897e-02,
                    2.43170841e-02,
                    8.06009483e-04,
                    0.00000000e00,
                    0.00000000e00,
                    0.00000000e00,
                    0.00000000e00,
                    0.00000000e00,
                ]
            ),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            kernel_lanczos(np.linspace(-5, 5, 25), 1),
            np.array(
                [
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.03647563,
                    0.54451556,
                    1.00000000,
                    0.54451556,
                    0.03647563,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                ]
            ),
            decimal=7,
        )


class TestKernelCardinalSpline(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.kernel_cardinal_spline`
    definition unit tests methods.
    """

    def test_kernel_cardinal_spline(self):
        """
        Test :func:`colour.algebra.interpolation.kernel_cardinal_spline`
        definition.
        """

        np.testing.assert_almost_equal(
            kernel_cardinal_spline(np.linspace(-5, 5, 25)),
            np.array(
                [
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    -0.03703704,
                    -0.0703125,
                    0.13194444,
                    0.67447917,
                    1.00000000,
                    0.67447917,
                    0.13194444,
                    -0.0703125,
                    -0.03703704,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                ]
            ),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            kernel_cardinal_spline(np.linspace(-5, 5, 25), 0, 1),
            np.array(
                [
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00617284,
                    0.0703125,
                    0.26157407,
                    0.52922454,
                    0.66666667,
                    0.52922454,
                    0.26157407,
                    0.0703125,
                    0.00617284,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                    0.00000000,
                ]
            ),
            decimal=7,
        )


class TestKernelInterpolator(unittest.TestCase):
    """
    Define :class:`colour.algebra.interpolation.KernelInterpolator` class unit
    tests methods.
    """

    def test_required_attributes(self):
        """Test the presence of required attributes."""

        required_attributes = (
            "x",
            "y",
            "window",
            "kernel",
            "kernel_kwargs",
            "padding_kwargs",
        )

        for attribute in required_attributes:
            self.assertIn(attribute, dir(KernelInterpolator))

    def test_required_methods(self):
        """Test the presence of required methods."""

        required_methods = ("__init__", "__call__")

        for method in required_methods:  # pragma: no cover
            self.assertIn(method, dir(KernelInterpolator))

    def test_x(self):
        """
        Test :attr:`colour.algebra.interpolation.KernelInterpolator.x`
        property.
        """

        x = y = np.linspace(0, 1, 10)
        kernel_interpolator = KernelInterpolator(x, y)

        np.testing.assert_equal(kernel_interpolator.x, x)

    def test_y(self):
        """
        Test :attr:`colour.algebra.interpolation.KernelInterpolator.y`
        property.
        """

        x = y = np.linspace(0, 1, 10)
        kernel_interpolator = KernelInterpolator(x, y)

        np.testing.assert_equal(kernel_interpolator.y, y)

    def test_window(self):
        """
        Test :attr:`colour.algebra.interpolation.KernelInterpolator.window`
        property.
        """

        x = y = np.linspace(0, 1, 10)
        kernel_interpolator = KernelInterpolator(x, y, window=3)

        self.assertEqual(kernel_interpolator.window, 3)

    def test_kernel(self):
        """
        Test :attr:`colour.algebra.interpolation.KernelInterpolator.kernel`
        property.
        """

        x = y = np.linspace(0, 1, 10)
        kernel_interpolator = KernelInterpolator(x, y, kernel=kernel_linear)

        self.assertIs(kernel_interpolator.kernel, kernel_linear)

    def test_kernel_kwargs(self):
        """
        Test :attr:`colour.algebra.interpolation.KernelInterpolator.\
kernel_kwargs` property.
        """

        x = y = np.linspace(0, 1, 10)
        kernel_kwargs = {"a": 1}
        kernel_interpolator = KernelInterpolator(
            x, y, kernel_kwargs=kernel_kwargs
        )

        self.assertDictEqual(kernel_interpolator.kernel_kwargs, kernel_kwargs)

    def test_padding_kwargs(self):
        """
        Test :attr:`colour.algebra.interpolation.KernelInterpolator.\
padding_kwargs` property.
        """

        x = y = np.linspace(0, 1, 10)
        padding_kwargs = {"pad_width": (3, 3), "mode": "mean"}
        kernel_interpolator = KernelInterpolator(
            x, y, padding_kwargs=padding_kwargs
        )

        self.assertDictEqual(
            kernel_interpolator.padding_kwargs, padding_kwargs
        )

    def test_raise_exception___init__(self):
        """
        Test :meth:`colour.algebra.interpolation.KernelInterpolator.__init__`
        method raised exception.
        """

        self.assertRaises(
            ValueError,
            KernelInterpolator,
            np.linspace(0, 1, 10),
            np.linspace(0, 1, 15),
        )

    def test__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.KernelInterpolator.__call__`
        method.
        """

        x = np.arange(11, 26, 1)
        y = np.sin(x / len(x) * np.pi * 6) / (x / len(x)) + np.pi
        x_i = np.linspace(11, 25, 25)

        kernel_interpolator = KernelInterpolator(x, y)
        np.testing.assert_almost_equal(
            kernel_interpolator(x_i),
            np.array(
                [
                    4.43848790,
                    4.26286480,
                    3.64640076,
                    2.77982023,
                    2.13474499,
                    2.08206794,
                    2.50585862,
                    3.24992692,
                    3.84593162,
                    4.06289704,
                    3.80825633,
                    3.21068994,
                    2.65177161,
                    2.32137382,
                    2.45995375,
                    2.88799997,
                    3.43843598,
                    3.79504892,
                    3.79937086,
                    3.47673343,
                    2.99303182,
                    2.59305006,
                    2.47805594,
                    2.82957843,
                    3.14159265,
                ]
            ),
            decimal=7,
        )

        kernel_interpolator = KernelInterpolator(x, y, kernel=kernel_sinc)
        np.testing.assert_almost_equal(
            kernel_interpolator(x_i),
            np.array(
                [
                    4.43848790,
                    4.47570010,
                    3.84353906,
                    3.05959493,
                    2.53514958,
                    2.19916874,
                    2.93225625,
                    3.32187855,
                    4.09458791,
                    4.23088094,
                    3.92591447,
                    3.53263071,
                    2.65177161,
                    2.73541557,
                    2.65740315,
                    3.17077616,
                    3.69624479,
                    3.87159620,
                    4.06433758,
                    3.56283868,
                    3.28312289,
                    2.79652091,
                    2.62481419,
                    3.22117115,
                    3.14159265,
                ]
            ),
            decimal=7,
        )

        kernel_interpolator = KernelInterpolator(x, y, window=1)
        np.testing.assert_almost_equal(
            kernel_interpolator(x_i),
            np.array(
                [
                    4.43848790,
                    4.96712277,
                    4.09584229,
                    3.23991575,
                    2.80418924,
                    2.28470276,
                    3.20024753,
                    3.41120944,
                    4.46416970,
                    4.57878168,
                    4.15371498,
                    3.92841633,
                    2.65177161,
                    3.02110187,
                    2.79812654,
                    3.44218674,
                    4.00032377,
                    4.01356870,
                    4.47633386,
                    3.70912627,
                    3.58365067,
                    3.14325415,
                    2.88247572,
                    3.37531662,
                    3.14159265,
                ]
            ),
            decimal=7,
        )

        kernel_interpolator = KernelInterpolator(
            x, y, window=1, kernel_kwargs={"a": 1}
        )
        np.testing.assert_almost_equal(
            kernel_interpolator(x_i),
            np.array(
                [
                    4.43848790,
                    3.34379320,
                    3.62463711,
                    2.34585418,
                    2.04767083,
                    2.09444849,
                    2.13349835,
                    3.10304927,
                    3.29553153,
                    3.59884738,
                    3.48484031,
                    2.72974983,
                    2.65177161,
                    2.03850468,
                    2.29470194,
                    2.76179863,
                    2.80189050,
                    3.75979450,
                    2.98422257,
                    3.48444099,
                    2.49208997,
                    2.46516442,
                    2.42336082,
                    2.25975903,
                    3.14159265,
                ]
            ),
            decimal=7,
        )

        kernel_interpolator = KernelInterpolator(
            x, y, padding_kwargs={"pad_width": (3, 3), "mode": "mean"}
        )
        np.testing.assert_almost_equal(
            kernel_interpolator(x_i),
            np.array(
                [
                    4.4384879,
                    4.35723245,
                    3.62918155,
                    2.77471295,
                    2.13474499,
                    2.08206794,
                    2.50585862,
                    3.24992692,
                    3.84593162,
                    4.06289704,
                    3.80825633,
                    3.21068994,
                    2.65177161,
                    2.32137382,
                    2.45995375,
                    2.88799997,
                    3.43843598,
                    3.79504892,
                    3.79937086,
                    3.47673343,
                    2.99303182,
                    2.59771985,
                    2.49380017,
                    2.76339043,
                    3.14159265,
                ]
            ),
            decimal=7,
        )

        x_1 = np.arange(1, 10, 1)
        x_2 = x_1 * 10
        x_3 = x_1 / 10
        y = np.sin(x_1 / len(x_1) * np.pi * 6) / (x_1 / len(x_1))
        x_i = np.linspace(1, 9, 25)

        np.testing.assert_almost_equal(
            KernelInterpolator(x_1, y)(x_i),
            KernelInterpolator(x_2, y)(x_i * 10),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            KernelInterpolator(x_1, y)(x_i),
            KernelInterpolator(x_3, y)(x_i / 10),
            decimal=7,
        )

    def test_raise_exception___call__(self):
        """
        Test :meth:`colour.algebra.interpolation.KernelInterpolator.__call__`
        method raised exception.
        """

        x = y = np.linspace(0, 1, 10)
        kernel_interpolator = KernelInterpolator(x, y)

        self.assertRaises(ValueError, kernel_interpolator, -1)

        self.assertRaises(ValueError, kernel_interpolator, 11)

    @ignore_numpy_errors
    def test_nan__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.KernelInterpolator.__call__`
        method nan support.
        """

        # NOTE: As the "x" independent variable must be uniform, it cannot
        # contain NaNs.
        # TODO: Revisit if the interpolator can be applied on non-uniform "x"
        # independent variable.

        pass


class TestNearestNeighbourInterpolator(unittest.TestCase):
    """
    Define :class:`colour.algebra.interpolation.NearestNeighbourInterpolator`
    class unit tests methods.
    """

    def test_required_attributes(self):
        """Test the presence of required attributes."""

        required_attributes = ()

        for attribute in required_attributes:  # pragma: no cover
            self.assertIn(attribute, dir(NearestNeighbourInterpolator))

    def test_required_methods(self):
        """Test the presence of required methods."""

        required_methods = ("__init__",)

        for method in required_methods:  # pragma: no cover
            self.assertIn(method, dir(NearestNeighbourInterpolator))

    def test___init__(self):
        """
        Test :meth:`colour.algebra.interpolation.KernelInterpolator.__init__`
        method.
        """

        x = y = np.linspace(0, 1, 10)
        nearest_neighbour_interpolator = NearestNeighbourInterpolator(
            x, y, kernel_kwargs={"a": 1}
        )

        self.assertDictEqual(nearest_neighbour_interpolator.kernel_kwargs, {})


class TestLinearInterpolator(unittest.TestCase):
    """
    Define :class:`colour.algebra.interpolation.LinearInterpolator` class unit
    tests methods.
    """

    def test_required_attributes(self):
        """Test the presence of required attributes."""

        required_attributes = ("x", "y")

        for attribute in required_attributes:
            self.assertIn(attribute, dir(LinearInterpolator))

    def test_required_methods(self):
        """Test the presence of required methods."""

        required_methods = ("__init__", "__call__")

        for method in required_methods:  # pragma: no cover
            self.assertIn(method, dir(LinearInterpolator))

    def test_raise_exception___init__(self):
        """
        Test :meth:`colour.algebra.interpolation.LinearInterpolator.__init__`
        method raised exception.
        """

        x, y = np.linspace(0, 1, 10), np.linspace(0, 1, 15)
        self.assertRaises(ValueError, LinearInterpolator, x, y)

    def test__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.LinearInterpolator.__call__`
        method.
        """

        interval = 0.1
        x = np.arange(len(DATA_POINTS_A))
        linear_interpolator = LinearInterpolator(x, DATA_POINTS_A)

        for i, value in enumerate(
            np.arange(0, len(DATA_POINTS_A) - 1 + interval, interval)
        ):
            self.assertAlmostEqual(
                DATA_POINTS_A_LINEAR_INTERPOLATED_10_SAMPLES[i],
                linear_interpolator(value),
                places=7,
            )

        np.testing.assert_almost_equal(
            linear_interpolator(
                np.arange(0, len(DATA_POINTS_A) - 1 + interval, interval)
            ),
            DATA_POINTS_A_LINEAR_INTERPOLATED_10_SAMPLES,
        )

    def test_raise_exception___call__(self):
        """
        Test :meth:`colour.algebra.interpolation.LinearInterpolator.__call__`
        method raised exception.
        """

        x = y = np.linspace(0, 1, 10)
        linear_interpolator = LinearInterpolator(x, y)

        self.assertRaises(ValueError, linear_interpolator, -1)

        self.assertRaises(ValueError, linear_interpolator, 11)

    @ignore_numpy_errors
    def test_nan__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.LinearInterpolator.__call__`
        method nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            try:
                linear_interpolator = LinearInterpolator(
                    np.array(case), np.array(case)
                )
                linear_interpolator(case[0])
            except ValueError:
                pass


class TestSpragueInterpolator(unittest.TestCase):
    """
    Define :class:`colour.algebra.interpolation.SpragueInterpolator` class
    unit tests methods.
    """

    def test_required_attributes(self):
        """Test the presence of required attributes."""

        required_attributes = ("x", "y")

        for attribute in required_attributes:
            self.assertIn(attribute, dir(SpragueInterpolator))

    def test_required_methods(self):
        """Test the presence of required methods."""

        required_methods = ("__init__", "__call__")

        for method in required_methods:  # pragma: no cover
            self.assertIn(method, dir(SpragueInterpolator))

    def test_raise_exception___init__(self):
        """
        Test :meth:`colour.algebra.interpolation.SpragueInterpolator.__init__`
        method raised exception.
        """

        x, y = np.linspace(0, 1, 10), np.linspace(0, 1, 15)
        self.assertRaises(ValueError, SpragueInterpolator, x, y)

    def test__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.SpragueInterpolator.__call__`
        method.
        """

        interval = 0.1
        x = np.arange(len(DATA_POINTS_A))
        sprague_interpolator = SpragueInterpolator(x, DATA_POINTS_A)

        for i, value in enumerate(
            np.arange(0, len(DATA_POINTS_A) - 1 + interval, interval)
        ):
            self.assertAlmostEqual(
                DATA_POINTS_A_SPRAGUE_INTERPOLATED_10_SAMPLES[i],
                sprague_interpolator(value),
                places=7,
            )

        np.testing.assert_almost_equal(
            sprague_interpolator(
                np.arange(0, len(DATA_POINTS_A) - 1 + interval, interval)
            ),
            DATA_POINTS_A_SPRAGUE_INTERPOLATED_10_SAMPLES,
        )

    def test_raise_exception___call__(self):
        """
        Test :meth:`colour.algebra.interpolation.SpragueInterpolator.__call__`
        method raised exception.
        """

        x = y = np.linspace(0, 1, 10)
        sprague_interpolator = SpragueInterpolator(x, y)

        self.assertRaises(ValueError, sprague_interpolator, -1)

        self.assertRaises(ValueError, sprague_interpolator, 11)

    @ignore_numpy_errors
    def test_nan__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.SpragueInterpolator.__call__`
        method nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            try:
                sprague_interpolator = SpragueInterpolator(
                    np.array(case), np.array(case)
                )
                sprague_interpolator(case[0])  # pragma: no cover
            except AssertionError:
                pass


class TestCubicSplineInterpolator(unittest.TestCase):
    """
    Define :class:`colour.algebra.interpolation.CubicSplineInterpolator` class
    unit tests methods.
    """

    def test__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.CubicSplineInterpolator.\
__call__` method.

        Notes
        -----
        -   This class is a wrapper around *scipy.interpolate.interp1d* class
            and is assumed to be unit tested thoroughly.
        """

        np.testing.assert_almost_equal(
            CubicSplineInterpolator(
                np.linspace(0, 1, len(DATA_POINTS_A)), DATA_POINTS_A
            )(np.linspace(0, 1, len(DATA_POINTS_A) * 2)),
            DATA_POINTS_A_CUBIC_SPLINE_INTERPOLATED_X2_SAMPLES,
        )


class TestPchipInterpolator(unittest.TestCase):
    """
    Define :class:`colour.algebra.interpolation.PchipInterpolator` class
    unit tests methods.
    """

    def test_required_attributes(self):
        """Test the presence of required attributes."""

        required_attributes = ("x", "y")

        for attribute in required_attributes:
            self.assertIn(attribute, dir(PchipInterpolator))

    def test_required_methods(self):
        """Test the presence of required methods."""

        required_methods = ("__init__",)

        for method in required_methods:  # pragma: no cover
            self.assertIn(method, dir(PchipInterpolator))


class TestNullInterpolator(unittest.TestCase):
    """
    Define :class:`colour.algebra.interpolation.NullInterpolator` class
    unit tests methods.
    """

    def test_required_attributes(self):
        """Test the presence of required attributes."""

        required_attributes = ("x", "y")

        for attribute in required_attributes:
            self.assertIn(attribute, dir(NullInterpolator))

    def test_required_methods(self):
        """Test the presence of required methods."""

        required_methods = ("__init__", "__call__")

        for method in required_methods:  # pragma: no cover
            self.assertIn(method, dir(NullInterpolator))

    def test_x(self):
        """
        Test :attr:`colour.algebra.interpolation.NullInterpolator.x`
        property.
        """

        x = y = np.linspace(0, 1, 10)
        null_interpolator = NullInterpolator(x, y)

        np.testing.assert_equal(null_interpolator.x, x)

    def test_y(self):
        """
        Test :attr:`colour.algebra.interpolation.NullInterpolator.y`
        property.
        """

        x = y = np.linspace(0, 1, 10)
        null_interpolator = NullInterpolator(x, y)

        np.testing.assert_equal(null_interpolator.y, y)

    def test_absolute_tolerance(self):
        """
        Test :attr:`colour.algebra.interpolation.NullInterpolator.\
absolute_tolerance` property.
        """

        x = y = np.linspace(0, 1, 10)
        null_interpolator = NullInterpolator(x, y, absolute_tolerance=0.1)

        np.testing.assert_equal(null_interpolator.absolute_tolerance, 0.1)

    def test_relative_tolerance(self):
        """
        Test :attr:`colour.algebra.interpolation.NullInterpolator.\
relative_tolerance` property.
        """

        x = y = np.linspace(0, 1, 10)
        null_interpolator = NullInterpolator(x, y, relative_tolerance=0.1)

        np.testing.assert_equal(null_interpolator.relative_tolerance, 0.1)

    def test_default(self):
        """
        Test :attr:`colour.algebra.interpolation.NullInterpolator.\
default` property.
        """

        x = y = np.linspace(0, 1, 10)
        null_interpolator = NullInterpolator(x, y, default=0)

        np.testing.assert_equal(null_interpolator.default, 0)

    def test_raise_exception___init__(self):
        """
        Test :meth:`colour.algebra.interpolation.NullInterpolator.__init__`
        method raised exception.
        """

        x, y = np.linspace(0, 1, 10), np.linspace(0, 1, 15)
        self.assertRaises(ValueError, NullInterpolator, x, y)

    def test__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.NullInterpolator.__call__`
        method.
        """

        x = np.arange(len(DATA_POINTS_A))
        null_interpolator = NullInterpolator(x, DATA_POINTS_A)
        np.testing.assert_almost_equal(
            null_interpolator(np.array([0.75, 2.0, 3.0, 4.75])),
            np.array([np.nan, 12.46, 9.51, np.nan]),
        )

        null_interpolator = NullInterpolator(x, DATA_POINTS_A, 0.25, 0.25)
        np.testing.assert_almost_equal(
            null_interpolator(np.array([0.75, 2.0, 3.0, 4.75])),
            np.array([12.32, 12.46, 9.51, 4.33]),
        )

    def test_raise_exception___call__(self):
        """
        Test :meth:`colour.algebra.interpolation.NullInterpolator.__call__`
        method raised exception.
        """

        x = y = np.linspace(0, 1, 10)
        null_interpolator = NullInterpolator(x, y)

        self.assertRaises(ValueError, null_interpolator, -1)

        self.assertRaises(ValueError, null_interpolator, 11)

    @ignore_numpy_errors
    def test_nan__call__(self):
        """
        Test :meth:`colour.algebra.interpolation.NullInterpolator.__call__`
        method nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            try:
                null_interpolator = NullInterpolator(
                    np.array(case), np.array(case)
                )
                null_interpolator(case[0])
            except ValueError:
                pass


class TestLagrangeCoefficients(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.lagrange_coefficients`
    definition unit tests methods.
    """

    def test_lagrange_coefficients(self):
        """
        Test :func:`colour.algebra.interpolation.lagrange_coefficients`
        definition.

        Notes
        -----
        -   :attr:`LAGRANGE_COEFFICIENTS_A` and :attr:`LAGRANGE_COEFFICIENTS_B`
            attributes data is matching :cite:`Fairman1985b`.

        References
        ----------
        :cite:`Fairman1985b`
        """

        lc = [lagrange_coefficients(i, 3) for i in np.linspace(0.05, 0.95, 19)]
        np.testing.assert_almost_equal(lc, LAGRANGE_COEFFICIENTS_A, decimal=7)

        lc = [lagrange_coefficients(i, 4) for i in np.linspace(1.05, 1.95, 19)]
        np.testing.assert_almost_equal(lc, LAGRANGE_COEFFICIENTS_B, decimal=7)


class TestVerticesAndRelativeCoordinates(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.\
vertices_and_relative_coordinates` definition unit tests methods.
    """

    def test_vertices_and_relative_coordinates(self):
        """
        Test :func:`colour.algebra.interpolation.\
vertices_and_relative_coordinates` definition.
        """

        prng = np.random.RandomState(4)

        V_xyz = random_triplet_generator(4, random_state=prng)
        vertices, V_xyzr = vertices_and_relative_coordinates(V_xyz, LUT_TABLE)

        np.testing.assert_almost_equal(
            vertices,
            np.array(
                [
                    [
                        [0.58919500, 0.58919500, 0.13916400],
                        [0.33333300, 0.00000000, 0.33333300],
                        [0.83331100, 0.83331100, 0.83331100],
                        [0.79789400, -0.03541200, -0.03541200],
                    ],
                    [
                        [0.59460100, 0.59460100, 0.36958600],
                        [0.39062300, 0.00000000, 0.78124600],
                        [0.83331100, 0.83331100, 1.24996300],
                        [0.75276700, -0.02847900, 0.36214400],
                    ],
                    [
                        [0.66343200, 0.93018800, 0.12992000],
                        [0.41665500, 0.41665500, 0.41665500],
                        [0.70710200, 1.11043500, 0.70710200],
                        [0.63333300, 0.31666700, 0.00000000],
                    ],
                    [
                        [0.68274900, 0.99108200, 0.37441600],
                        [0.41665500, 0.41665500, 0.83330800],
                        [0.51971400, 0.74472900, 0.74472900],
                        [0.73227800, 0.31562600, 0.31562600],
                    ],
                    [
                        [0.89131800, 0.61982300, 0.07683300],
                        [0.75276700, -0.02847900, 0.36214400],
                        [1.06561000, 0.64895700, 0.64895700],
                        [1.19684100, -0.05311700, -0.05311700],
                    ],
                    [
                        [0.95000000, 0.63333300, 0.31666700],
                        [0.66666700, 0.00000000, 0.66666700],
                        [1.00000000, 0.66666700, 1.00000000],
                        [1.16258800, -0.05037200, 0.35394800],
                    ],
                    [
                        [0.88379200, 0.88379200, 0.20874600],
                        [0.73227800, 0.31562600, 0.31562600],
                        [0.89460600, 0.89460600, 0.66959000],
                        [1.03843900, 0.31089900, -0.05287000],
                    ],
                    [
                        [0.88919900, 0.88919900, 0.43916800],
                        [0.66666700, 0.33333300, 0.66666700],
                        [1.24996600, 1.24996600, 1.24996600],
                        [1.13122500, 0.29792000, 0.29792000],
                    ],
                ]
            ),
        )
        np.testing.assert_almost_equal(
            V_xyzr,
            np.array(
                [
                    [0.90108952, 0.09318647, 0.75894709],
                    [0.64169675, 0.64826849, 0.30437460],
                    [0.91805308, 0.92882336, 0.33814877],
                    [0.14444798, 0.01869077, 0.59305522],
                ]
            ),
        )


class TestTableInterpolationTrilinear(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.\
table_interpolation_trilinear` definition unit tests methods.
    """

    def test_interpolation_trilinear(self):
        """
        Test :func:`colour.algebra.interpolation.\
table_interpolation_trilinear` definition.
        """

        prng = np.random.RandomState(4)

        V_xyz = random_triplet_generator(16, random_state=prng)

        np.testing.assert_almost_equal(
            table_interpolation_trilinear(V_xyz, LUT_TABLE),
            np.array(
                [
                    [1.07937594, -0.02773926, 0.55498254],
                    [0.53983424, 0.37099516, 0.13994561],
                    [1.13449122, -0.00305380, 0.13792909],
                    [0.73411897, 1.00141020, 0.59348239],
                    [0.74066176, 0.44679540, 0.55030394],
                    [0.20634750, 0.84797880, 0.55905579],
                    [0.92348649, 0.73112515, 0.42362820],
                    [0.03639248, 0.70357649, 0.52375041],
                    [0.29215488, 0.19697840, 0.44603879],
                    [0.47793470, 0.08696360, 0.70288463],
                    [0.88883354, 0.68680856, 0.87404642],
                    [0.21430977, 0.16796653, 0.19634247],
                    [0.82118989, 0.69239283, 0.39932389],
                    [1.06679072, 0.37974319, 0.49759377],
                    [0.17856230, 0.44755467, 0.62045271],
                    [0.59220355, 0.93136492, 0.30063692],
                ]
            ),
        )


class TestTableInterpolationTetrahedral(unittest.TestCase):
    """
    Define :func:`colour.algebra.interpolation.\
table_interpolation_tetrahedral` definition unit tests methods.
    """

    def test_interpolation_tetrahedral(self):
        """
        Test :func:`colour.algebra.interpolation.\
table_interpolation_tetrahedral` definition.
        """

        prng = np.random.RandomState(4)

        V_xyz = random_triplet_generator(16, random_state=prng)

        np.testing.assert_almost_equal(
            table_interpolation_tetrahedral(V_xyz, LUT_TABLE),
            np.array(
                [
                    [1.08039215, -0.02840092, 0.55855303],
                    [0.52208945, 0.35297753, 0.13599555],
                    [1.14373467, -0.00422138, 0.13413290],
                    [0.71384967, 0.98420883, 0.57982724],
                    [0.76771576, 0.46280975, 0.55106736],
                    [0.20861663, 0.85077712, 0.57102264],
                    [0.90398698, 0.72351675, 0.41151955],
                    [0.03749453, 0.70226823, 0.52614254],
                    [0.29626758, 0.21645072, 0.47615873],
                    [0.46729624, 0.07494851, 0.68892548],
                    [0.85907681, 0.67744258, 0.84410486],
                    [0.24335535, 0.20896545, 0.21996717],
                    [0.79244027, 0.66930773, 0.39213595],
                    [1.08383608, 0.37985897, 0.49011919],
                    [0.14683649, 0.43624903, 0.58706947],
                    [0.61272658, 0.92799297, 0.29650424],
                ]
            ),
        )


if __name__ == "__main__":
    unittest.main()
